

import pyflow_acdc as pyf
import pandas as pd

"""
Converted to PyFlowACDC format from

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%                                                                  %%%%%
%%%%    IEEE PES Power Grid Library - Optimal Power Flow - v21.07     %%%%%
%%%%          (https://github.com/power-grid-lib/pglib-opf)           %%%%%
%%%%               Benchmark Group - Typical Operations               %%%%%
%%%%                         29 - July - 2021                         %%%%%
%%%%                                                                  %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Power flow data for IEEE 300 bus test case.
%   This data was converted from IEEE Common Data Format
%   (ieee300cdf.txt) on 20-Sep-2004 by cdf2matp, rev. 1.11
%
%   Converted from IEEE CDF file from:
%       http://www.ee.washington.edu/research/pstca/
%
%   Copyright (c) 1999 by Richard D. Christie, University of Washington
%   Electrical Engineering Licensed under the Creative Commons Attribution 4.0
%   International license, http://creativecommons.org/licenses/by/4.0/
%
%   CDF Header:
%   13/05/91 CYME INTERNATIONAL    100.0 1991 S IEEE 300-BUS TEST SYSTEM
%
"""
def pglib_opf_case300_ieee():    
    
    S_base=100
    
    # DataFrame Code:
    nodes_AC_data = [
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.9, 'Reactive_load': 0.49, 'Node_id': '1.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.56, 'Reactive_load': 0.15, 'Node_id': '2.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2, 'Reactive_load': 0.0, 'Node_id': '3.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '4.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 3.53, 'Reactive_load': 1.3, 'Node_id': '5.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2, 'Reactive_load': 0.41, 'Node_id': '6.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.63, 'Reactive_load': 0.14, 'Node_id': '8.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.96, 'Reactive_load': 0.43, 'Node_id': '9.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.53, 'Reactive_load': 0.33, 'Node_id': '10.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.83, 'Reactive_load': 0.21, 'Node_id': '11.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '12.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.58, 'Reactive_load': 0.1, 'Node_id': '13.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.6, 'Reactive_load': 0.6, 'Node_id': '14.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.2670000000000001, 'Reactive_load': 0.23, 'Node_id': '15.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '16.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 5.61, 'Reactive_load': 2.2, 'Node_id': '17.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '19.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 6.05, 'Reactive_load': 1.2, 'Node_id': '20.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.77, 'Reactive_load': 0.01, 'Node_id': '21.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.81, 'Reactive_load': 0.23, 'Node_id': '22.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.21, 'Reactive_load': 0.07, 'Node_id': '23.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '24.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.45, 'Reactive_load': 0.12, 'Node_id': '25.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.28, 'Reactive_load': 0.09, 'Node_id': '26.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.69, 'Reactive_load': 0.13, 'Node_id': '27.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.55, 'Reactive_load': 0.06, 'Node_id': '33.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '34.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '35.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '36.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.85, 'Reactive_load': 0.32, 'Node_id': '37.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.55, 'Reactive_load': 0.18, 'Node_id': '38.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '39.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.46, 'Reactive_load': -0.21, 'Node_id': '40.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.86, 'Reactive_load': 0.0, 'Node_id': '41.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '42.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.39, 'Reactive_load': 0.09, 'Node_id': '43.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.95, 'Reactive_load': 0.29, 'Node_id': '44.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '45.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '46.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.58, 'Reactive_load': 0.11800000000000001, 'Node_id': '47.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.41, 'Reactive_load': 0.19, 'Node_id': '48.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.92, 'Reactive_load': 0.26, 'Node_id': '49.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.05, 'Reactive_load': 0.05, 'Node_id': '51.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.61, 'Reactive_load': 0.28, 'Node_id': '52.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.69, 'Reactive_load': 0.03, 'Node_id': '53.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1, 'Reactive_load': 0.01, 'Node_id': '54.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.22, 'Reactive_load': 0.1, 'Node_id': '55.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.98, 'Reactive_load': 0.2, 'Node_id': '57.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.14, 'Reactive_load': 0.01, 'Node_id': '58.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.18, 'Reactive_load': 1.06, 'Node_id': '59.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '60.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.27, 'Reactive_load': 1.1, 'Node_id': '61.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '62.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7, 'Reactive_load': 0.3, 'Node_id': '63.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '64.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '69.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.56, 'Reactive_load': 0.2, 'Node_id': '70.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.16, 'Reactive_load': 0.38, 'Node_id': '71.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.57, 'Reactive_load': 0.19, 'Node_id': '72.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.24, 'Reactive_load': 0.71, 'Node_id': '73.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '74.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.08, 'Reactive_load': 1.07, 'Node_id': '76.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.74, 'Reactive_load': 0.28, 'Node_id': '77.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '78.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.48, 'Reactive_load': 0.14, 'Node_id': '79.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.28, 'Reactive_load': 0.07, 'Node_id': '80.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '81.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.37, 'Reactive_load': 0.13, 'Node_id': '84.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '85.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '86.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '87.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '88.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.442, 'Reactive_load': 0.0, 'Node_id': '89.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.66, 'Reactive_load': 0.0, 'Node_id': '90.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.174, 'Reactive_load': 0.0, 'Node_id': '91.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.158, 'Reactive_load': 0.0, 'Node_id': '92.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.603, 'Reactive_load': 0.0, 'Node_id': '94.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.39899999999999997, 'Reactive_load': 0.0, 'Node_id': '97.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.667, 'Reactive_load': 0.0, 'Node_id': '98.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.835, 'Reactive_load': 0.0, 'Node_id': '99.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '100.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.778, 'Reactive_load': 0.0, 'Node_id': '102.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.32, 'Reactive_load': 0.0, 'Node_id': '103.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.086, 'Reactive_load': 0.0, 'Node_id': '104.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.496, 'Reactive_load': 0.0, 'Node_id': '105.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.046, 'Reactive_load': 0.0, 'Node_id': '107.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.121, 'Reactive_load': 0.0, 'Node_id': '108.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.307, 'Reactive_load': 0.0, 'Node_id': '109.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.63, 'Reactive_load': 0.0, 'Node_id': '110.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.196, 'Reactive_load': 0.0, 'Node_id': '112.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.262, 'Reactive_load': 0.0, 'Node_id': '113.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.182, 'Reactive_load': 0.0, 'Node_id': '114.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '115.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '116.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '117.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 3.25, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.141, 'Reactive_load': 6.5, 'Node_id': '118.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '119.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 7.77, 'Reactive_load': 2.15, 'Node_id': '120.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.55, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 5.35, 'Reactive_load': 0.55, 'Node_id': '121.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.291, 'Reactive_load': 0.11800000000000001, 'Node_id': '122.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.78, 'Reactive_load': 0.013999999999999999, 'Node_id': '123.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.764, 'Reactive_load': 0.593, 'Node_id': '124.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 5.148, 'Reactive_load': 0.8270000000000001, 'Node_id': '125.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.579, 'Reactive_load': 0.051, 'Node_id': '126.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 3.8080000000000003, 'Reactive_load': 0.37, 'Node_id': '127.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '128.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '129.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '130.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '131.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '132.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '133.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '134.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.692, 'Reactive_load': 0.41600000000000004, 'Node_id': '135.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.552, 'Reactive_load': 0.182, 'Node_id': '136.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.736, 'Reactive_load': 0.998, 'Node_id': '137.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 10.192, 'Reactive_load': 1.3519999999999999, 'Node_id': '138.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 5.95, 'Reactive_load': 0.833, 'Node_id': '139.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 3.877, 'Reactive_load': 1.147, 'Node_id': '140.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.45, 'Reactive_load': 0.58, 'Node_id': '141.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.565, 'Reactive_load': 0.245, 'Node_id': '142.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.895, 'Reactive_load': 0.355, 'Node_id': '143.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '144.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.24, 'Reactive_load': 0.14, 'Node_id': '145.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '146.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '147.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.63, 'Reactive_load': 0.25, 'Node_id': '148.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '149.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '150.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '151.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.17, 'Reactive_load': 0.09, 'Node_id': '152.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '153.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7, 'Reactive_load': 0.05, 'Node_id': '154.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.345, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.0, 'Reactive_load': 0.5, 'Node_id': '155.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.75, 'Reactive_load': 0.5, 'Node_id': '156.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.235, 'Reactive_load': -0.243, 'Node_id': '157.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '158.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.33, 'Reactive_load': 0.165, 'Node_id': '159.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '160.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.35, 'Reactive_load': 0.15, 'Node_id': '161.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.85, 'Reactive_load': 0.24, 'Node_id': '162.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.004, 'Node_id': '163.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '164.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': -2.12, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '165.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '166.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': -1.03, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.9989999999999997, 'Reactive_load': 0.9570000000000001, 'Node_id': '167.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '168.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '169.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 4.8180000000000005, 'Reactive_load': 2.05, 'Node_id': '170.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 7.636, 'Reactive_load': 2.911, 'Node_id': '171.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.265, 'Reactive_load': 0.0, 'Node_id': '172.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.635, 'Reactive_load': 0.43, 'Node_id': '173.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.53, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '174.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.76, 'Reactive_load': 0.83, 'Node_id': '175.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.05, 'Reactive_load': 0.04, 'Node_id': '176.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.28, 'Reactive_load': 0.12, 'Node_id': '177.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 4.274, 'Reactive_load': 1.736, 'Node_id': '178.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.74, 'Reactive_load': 0.29, 'Node_id': '179.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.45, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.695, 'Reactive_load': 0.493, 'Node_id': '180.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.7340000000000001, 'Reactive_load': 0.0, 'Node_id': '181.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.407, 'Reactive_load': 0.89, 'Node_id': '182.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.4, 'Reactive_load': 0.04, 'Node_id': '183.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.368, 'Reactive_load': 0.166, 'Node_id': '184.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '185.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.598, 'Reactive_load': 0.243, 'Node_id': '186.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.598, 'Reactive_load': 0.243, 'Node_id': '187.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.8259999999999998, 'Reactive_load': 0.436, 'Node_id': '188.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07, 'Reactive_load': 0.02, 'Node_id': '189.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '190.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': -1.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 4.89, 'Reactive_load': 0.53, 'Node_id': '191.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 8.0, 'Reactive_load': 0.72, 'Node_id': '192.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '193.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '194.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '195.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.1, 'Reactive_load': 0.03, 'Node_id': '196.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.43, 'Reactive_load': 0.14, 'Node_id': '197.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.64, 'Reactive_load': 0.21, 'Node_id': '198.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.35, 'Reactive_load': 0.12, 'Node_id': '199.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.27, 'Reactive_load': 0.12, 'Node_id': '200.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.41, 'Reactive_load': 0.14, 'Node_id': '201.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.38, 'Reactive_load': 0.13, 'Node_id': '202.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.42, 'Reactive_load': 0.14, 'Node_id': '203.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.72, 'Reactive_load': 0.24, 'Node_id': '204.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': -0.05, 'Node_id': '205.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.12, 'Reactive_load': 0.02, 'Node_id': '206.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.21, 'Reactive_load': -0.142, 'Node_id': '207.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.07, 'Reactive_load': 0.02, 'Node_id': '208.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.38, 'Reactive_load': 0.13, 'Node_id': '209.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '210.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.96, 'Reactive_load': 0.07, 'Node_id': '211.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '212.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 16.5, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '213.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.22, 'Reactive_load': 0.16, 'Node_id': '214.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.47, 'Reactive_load': 0.26, 'Node_id': '215.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.76, 'Reactive_load': 1.05, 'Node_id': '216.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0, 'Reactive_load': 0.75, 'Node_id': '217.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.31, 'Reactive_load': 0.96, 'Node_id': '218.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '219.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.85, 'Reactive_load': 1.0, 'Node_id': '220.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.71, 'Reactive_load': 0.7, 'Node_id': '221.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 3.28, 'Reactive_load': 1.88, 'Node_id': '222.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 4.28, 'Reactive_load': 2.32, 'Node_id': '223.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.73, 'Reactive_load': 0.99, 'Node_id': '224.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 4.1, 'Reactive_load': 0.4, 'Node_id': '225.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '226.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 27.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 5.38, 'Reactive_load': 3.69, 'Node_id': '227.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.23, 'Reactive_load': 1.48, 'Node_id': '228.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.96, 'Reactive_load': 0.46, 'Node_id': '229.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '230.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.59, 'Reactive_load': 1.07, 'Node_id': '231.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': -3.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 4.48, 'Reactive_load': 1.43, 'Node_id': '232.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 4.04, 'Reactive_load': 2.12, 'Node_id': '233.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 5.72, 'Reactive_load': 2.44, 'Node_id': '234.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.69, 'Reactive_load': 1.57, 'Node_id': '235.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '236.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '237.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 2.55, 'Reactive_load': 1.49, 'Node_id': '238.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': -1.5, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '239.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '240.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': -1.4, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '241.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 138.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '242.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.08, 'Reactive_load': 0.03, 'Node_id': '243.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '244.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.61, 'Reactive_load': 0.3, 'Node_id': '245.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.77, 'Reactive_load': 0.33, 'Node_id': '246.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.61, 'Reactive_load': 0.3, 'Node_id': '247.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.29, 'Reactive_load': 0.14, 'Node_id': '248.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.456, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.29, 'Reactive_load': 0.14, 'Node_id': '249.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 66.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.23, 'Reactive_load': -0.17, 'Node_id': '250.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.331, 'Reactive_load': -0.294, 'Node_id': '281.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.158, 'Reactive_load': -0.24, 'Node_id': '319.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.024, 'Reactive_load': -0.126, 'Node_id': '320.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.024, 'Reactive_load': -0.039, 'Node_id': '322.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.149, 'Reactive_load': 0.265, 'Node_id': '323.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.247, 'Reactive_load': -0.012, 'Node_id': '324.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.453, 'Reactive_load': -0.349, 'Node_id': '526.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.281, 'Reactive_load': -0.205, 'Node_id': '528.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.14, 'Reactive_load': 0.025, 'Node_id': '531.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -0.111, 'Reactive_load': -0.013999999999999999, 'Node_id': '552.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 230.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.505, 'Reactive_load': 0.174, 'Node_id': '562.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.29600000000000004, 'Reactive_load': 0.006, 'Node_id': '609.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 345.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -1.137, 'Reactive_load': 0.767, 'Node_id': '664.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 86.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 1.0031, 'Reactive_load': 0.2917, 'Node_id': '1190.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 86.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': -1.0, 'Reactive_load': 0.3417, 'Node_id': '1200.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '1201.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '2040.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7001.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7002.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7003.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7011.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7012.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7017.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7023.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7024.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 20.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7039.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7044.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'Slack', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7049.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7055.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7057.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7061.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7062.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7071.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7130.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7139.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '7166.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9001.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.042, 'Reactive_load': 0.0, 'Node_id': '9002.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0271, 'Reactive_load': 0.009399999999999999, 'Node_id': '9003.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0014000000000000002, 'Bs': 0.024, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0086, 'Reactive_load': 0.0028000000000000004, 'Node_id': '9004.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 115.0, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9005.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9006.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9007.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9012.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0475, 'Reactive_load': 0.015600000000000001, 'Node_id': '9021.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.015300000000000001, 'Reactive_load': 0.0053, 'Node_id': '9022.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0008, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9023.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.013500000000000002, 'Reactive_load': 0.004699999999999999, 'Node_id': '9024.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0007000000000000001, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0045000000000000005, 'Reactive_load': 0.0016, 'Node_id': '9025.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0002, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0045000000000000005, 'Reactive_load': 0.0016, 'Node_id': '9026.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0002, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0184, 'Reactive_load': 0.0064, 'Node_id': '9031.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.001, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0139, 'Reactive_load': 0.0048, 'Node_id': '9032.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0007000000000000001, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0189, 'Reactive_load': 0.006500000000000001, 'Node_id': '9033.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.001, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0155, 'Reactive_load': 0.0054, 'Node_id': '9034.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0008, 'Bs': 0.0172, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0166, 'Reactive_load': 0.0058, 'Node_id': '9035.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0009, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 2.3, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.030299999999999997, 'Reactive_load': 0.01, 'Node_id': '9036.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.018600000000000002, 'Reactive_load': 0.0064, 'Node_id': '9037.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.001, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0258, 'Reactive_load': 0.0089, 'Node_id': '9038.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0014000000000000002, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0101, 'Reactive_load': 0.0034999999999999996, 'Node_id': '9041.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0005, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.008100000000000001, 'Reactive_load': 0.0028000000000000004, 'Node_id': '9042.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0004, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 2.3, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.016, 'Reactive_load': 0.0052, 'Node_id': '9043.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9044.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.35810000000000003, 'Reactive_load': 0.0, 'Node_id': '9051.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.3, 'Reactive_load': 0.23, 'Node_id': '9052.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.2648, 'Reactive_load': 0.0, 'Node_id': '9053.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9054.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PV', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 13.8, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0, 'Reactive_load': 0.0, 'Node_id': '9055.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0102, 'Reactive_load': 0.0034999999999999996, 'Node_id': '9071.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0005, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 0.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.0102, 'Reactive_load': 0.0034999999999999996, 'Node_id': '9072.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0005, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 6.6, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.038, 'Reactive_load': 0.0125, 'Node_id': '9121.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.0, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None},
        {'type': 'PQ', 'Voltage_0': 1.0, 'theta_0': 0.0, 'kV_base': 2.3, 'Power_Gained': 0, 'Reactive_Gained': 0, 'Power_load': 0.011899999999999999, 'Reactive_load': 0.0040999999999999995, 'Node_id': '9533.0', 'Umin': 0.94, 'Umax': 1.06, 'Gs': 0.001, 'Bs': 0.0, 'x_coord': None, 'y_coord': None, 'PZ': None}
    ]
    nodes_AC = pd.DataFrame(nodes_AC_data)

    lines_AC_data = [
        {'fromNode': '37.0', 'toNode': '9001.0', 'Resistance': 6e-05, 'Reactance': 0.00046, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 9900.0, 'kV_base': 115.0, 'm': 1.0082, 'shift': 0.0, 'Line_id': '1'},
        {'fromNode': '9001.0', 'toNode': '9005.0', 'Resistance': 0.0008, 'Reactance': 0.00348, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 8215.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '2'},
        {'fromNode': '9001.0', 'toNode': '9006.0', 'Resistance': 0.02439, 'Reactance': 0.43682, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 68.0, 'kV_base': 6.6, 'm': 0.9668, 'shift': 0.0, 'Line_id': '3'},
        {'fromNode': '9001.0', 'toNode': '9012.0', 'Resistance': 0.03624, 'Reactance': 0.64898, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 46.0, 'kV_base': 6.6, 'm': 0.9796, 'shift': 0.0, 'Line_id': '4'},
        {'fromNode': '9005.0', 'toNode': '9051.0', 'Resistance': 0.01578, 'Reactance': 0.37486, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 79.0, 'kV_base': 13.8, 'm': 1.0435, 'shift': 0.0, 'Line_id': '5'},
        {'fromNode': '9005.0', 'toNode': '9052.0', 'Resistance': 0.01578, 'Reactance': 0.37486, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 79.0, 'kV_base': 13.8, 'm': 0.9391, 'shift': 0.0, 'Line_id': '6'},
        {'fromNode': '9005.0', 'toNode': '9053.0', 'Resistance': 0.01602, 'Reactance': 0.38046, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 78.0, 'kV_base': 13.8, 'm': 1.0435, 'shift': 0.0, 'Line_id': '7'},
        {'fromNode': '9005.0', 'toNode': '9054.0', 'Resistance': 0.0, 'Reactance': 0.152, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 193.0, 'kV_base': 13.8, 'm': 1.0435, 'shift': 0.0, 'Line_id': '8'},
        {'fromNode': '9005.0', 'toNode': '9055.0', 'Resistance': 0.0, 'Reactance': 0.8, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 37.0, 'kV_base': 13.8, 'm': 1.0435, 'shift': 0.0, 'Line_id': '9'},
        {'fromNode': '9006.0', 'toNode': '9007.0', 'Resistance': 0.05558, 'Reactance': 0.24666, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 117.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '10'},
        {'fromNode': '9006.0', 'toNode': '9003.0', 'Resistance': 0.11118, 'Reactance': 0.49332, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 59.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '11'},
        {'fromNode': '9006.0', 'toNode': '9003.0', 'Resistance': 0.11118, 'Reactance': 0.49332, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 59.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '12'},
        {'fromNode': '9012.0', 'toNode': '9002.0', 'Resistance': 0.07622, 'Reactance': 0.43286, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 67.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '13'},
        {'fromNode': '9012.0', 'toNode': '9002.0', 'Resistance': 0.07622, 'Reactance': 0.43286, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 67.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '14'},
        {'fromNode': '9002.0', 'toNode': '9021.0', 'Resistance': 0.0537, 'Reactance': 0.07026, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 332.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '15'},
        {'fromNode': '9021.0', 'toNode': '9023.0', 'Resistance': 1.1068, 'Reactance': 0.95278, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 21.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '16'},
        {'fromNode': '9021.0', 'toNode': '9022.0', 'Resistance': 0.44364, 'Reactance': 2.8152, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 11.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '17'},
        {'fromNode': '9002.0', 'toNode': '9024.0', 'Resistance': 0.50748, 'Reactance': 3.2202, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 9.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '18'},
        {'fromNode': '9023.0', 'toNode': '9025.0', 'Resistance': 0.66688, 'Reactance': 3.944, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 8.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '19'},
        {'fromNode': '9023.0', 'toNode': '9026.0', 'Resistance': 0.6113, 'Reactance': 3.6152, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 8.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '20'},
        {'fromNode': '9007.0', 'toNode': '9071.0', 'Resistance': 0.4412, 'Reactance': 2.9668, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 10.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '21'},
        {'fromNode': '9007.0', 'toNode': '9072.0', 'Resistance': 0.30792, 'Reactance': 2.057, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 15.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '22'},
        {'fromNode': '9007.0', 'toNode': '9003.0', 'Resistance': 0.0558, 'Reactance': 0.24666, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 116.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '23'},
        {'fromNode': '9003.0', 'toNode': '9031.0', 'Resistance': 0.73633, 'Reactance': 4.6724, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 7.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '24'},
        {'fromNode': '9003.0', 'toNode': '9032.0', 'Resistance': 0.76978, 'Reactance': 4.8846, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 6.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '25'},
        {'fromNode': '9003.0', 'toNode': '9033.0', 'Resistance': 0.75732, 'Reactance': 4.8056, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 7.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '26'},
        {'fromNode': '9003.0', 'toNode': '9044.0', 'Resistance': 0.07378, 'Reactance': 0.06352, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 302.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '27'},
        {'fromNode': '9044.0', 'toNode': '9004.0', 'Resistance': 0.03832, 'Reactance': 0.02894, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 611.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '28'},
        {'fromNode': '9004.0', 'toNode': '9041.0', 'Resistance': 0.36614, 'Reactance': 2.456, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 12.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '29'},
        {'fromNode': '9004.0', 'toNode': '9042.0', 'Resistance': 1.0593, 'Reactance': 5.4536, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 6.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '30'},
        {'fromNode': '9004.0', 'toNode': '9043.0', 'Resistance': 0.1567, 'Reactance': 1.6994, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 18.0, 'kV_base': 2.3, 'm': 1.0, 'shift': 0.0, 'Line_id': '31'},
        {'fromNode': '9003.0', 'toNode': '9034.0', 'Resistance': 0.13006, 'Reactance': 1.3912, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 21.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '32'},
        {'fromNode': '9003.0', 'toNode': '9035.0', 'Resistance': 0.54484, 'Reactance': 3.4572, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 9.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '33'},
        {'fromNode': '9003.0', 'toNode': '9036.0', 'Resistance': 0.15426, 'Reactance': 1.6729, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 18.0, 'kV_base': 2.3, 'm': 1.0, 'shift': 0.0, 'Line_id': '34'},
        {'fromNode': '9003.0', 'toNode': '9037.0', 'Resistance': 0.3849, 'Reactance': 2.5712, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 12.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '35'},
        {'fromNode': '9003.0', 'toNode': '9038.0', 'Resistance': 0.4412, 'Reactance': 2.9668, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 10.0, 'kV_base': 0.6, 'm': 1.0, 'shift': 0.0, 'Line_id': '36'},
        {'fromNode': '9012.0', 'toNode': '9121.0', 'Resistance': 0.23552, 'Reactance': 0.99036, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 29.0, 'kV_base': 6.6, 'm': 1, 'shift': 0, 'Line_id': '37'},
        {'fromNode': '9053.0', 'toNode': '9533.0', 'Resistance': 0.0, 'Reactance': 0.75, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 40.0, 'kV_base': 2.3, 'm': 0.9583, 'shift': 0.0, 'Line_id': '38'},
        {'fromNode': '1.0', 'toNode': '5.0', 'Resistance': 0.001, 'Reactance': 0.006, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 4823.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '39'},
        {'fromNode': '2.0', 'toNode': '6.0', 'Resistance': 0.001, 'Reactance': 0.009, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 3240.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '40'},
        {'fromNode': '2.0', 'toNode': '8.0', 'Resistance': 0.006, 'Reactance': 0.027, 'Conductance': 0, 'Susceptance': 0.054, 'MVA_rating': 1061.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '41'},
        {'fromNode': '3.0', 'toNode': '7.0', 'Resistance': 0.0, 'Reactance': 0.003, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 9778.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '42'},
        {'fromNode': '3.0', 'toNode': '19.0', 'Resistance': 0.008, 'Reactance': 0.069, 'Conductance': 0, 'Susceptance': 0.139, 'MVA_rating': 423.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '43'},
        {'fromNode': '3.0', 'toNode': '150.0', 'Resistance': 0.001, 'Reactance': 0.007, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 4149.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '44'},
        {'fromNode': '4.0', 'toNode': '16.0', 'Resistance': 0.002, 'Reactance': 0.019, 'Conductance': 0, 'Susceptance': 1.127, 'MVA_rating': 1536.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '45'},
        {'fromNode': '5.0', 'toNode': '9.0', 'Resistance': 0.006, 'Reactance': 0.029, 'Conductance': 0, 'Susceptance': 0.018, 'MVA_rating': 991.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '46'},
        {'fromNode': '7.0', 'toNode': '12.0', 'Resistance': 0.001, 'Reactance': 0.009, 'Conductance': 0, 'Susceptance': 0.07, 'MVA_rating': 3240.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '47'},
        {'fromNode': '7.0', 'toNode': '131.0', 'Resistance': 0.001, 'Reactance': 0.007, 'Conductance': 0, 'Susceptance': 0.014, 'MVA_rating': 4149.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '48'},
        {'fromNode': '8.0', 'toNode': '11.0', 'Resistance': 0.013, 'Reactance': 0.0595, 'Conductance': 0, 'Susceptance': 0.033, 'MVA_rating': 482.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '49'},
        {'fromNode': '8.0', 'toNode': '14.0', 'Resistance': 0.013, 'Reactance': 0.042, 'Conductance': 0, 'Susceptance': 0.081, 'MVA_rating': 668.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '50'},
        {'fromNode': '9.0', 'toNode': '11.0', 'Resistance': 0.006, 'Reactance': 0.027, 'Conductance': 0, 'Susceptance': 0.013, 'MVA_rating': 1061.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '51'},
        {'fromNode': '11.0', 'toNode': '13.0', 'Resistance': 0.008, 'Reactance': 0.034, 'Conductance': 0, 'Susceptance': 0.018, 'MVA_rating': 840.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '52'},
        {'fromNode': '12.0', 'toNode': '21.0', 'Resistance': 0.002, 'Reactance': 0.015, 'Conductance': 0, 'Susceptance': 0.118, 'MVA_rating': 1939.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '53'},
        {'fromNode': '13.0', 'toNode': '20.0', 'Resistance': 0.006, 'Reactance': 0.034, 'Conductance': 0, 'Susceptance': 0.016, 'MVA_rating': 850.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '54'},
        {'fromNode': '14.0', 'toNode': '15.0', 'Resistance': 0.014, 'Reactance': 0.042, 'Conductance': 0, 'Susceptance': 0.097, 'MVA_rating': 663.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '55'},
        {'fromNode': '15.0', 'toNode': '37.0', 'Resistance': 0.065, 'Reactance': 0.248, 'Conductance': 0, 'Susceptance': 0.121, 'MVA_rating': 115.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '56'},
        {'fromNode': '15.0', 'toNode': '89.0', 'Resistance': 0.099, 'Reactance': 0.248, 'Conductance': 0, 'Susceptance': 0.035, 'MVA_rating': 110.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '57'},
        {'fromNode': '15.0', 'toNode': '90.0', 'Resistance': 0.096, 'Reactance': 0.363, 'Conductance': 0, 'Susceptance': 0.048, 'MVA_rating': 79.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '58'},
        {'fromNode': '16.0', 'toNode': '42.0', 'Resistance': 0.002, 'Reactance': 0.022, 'Conductance': 0, 'Susceptance': 1.28, 'MVA_rating': 1328.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '59'},
        {'fromNode': '19.0', 'toNode': '21.0', 'Resistance': 0.002, 'Reactance': 0.018, 'Conductance': 0, 'Susceptance': 0.036, 'MVA_rating': 1620.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '60'},
        {'fromNode': '19.0', 'toNode': '87.0', 'Resistance': 0.013, 'Reactance': 0.08, 'Conductance': 0, 'Susceptance': 0.151, 'MVA_rating': 362.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '61'},
        {'fromNode': '20.0', 'toNode': '22.0', 'Resistance': 0.016, 'Reactance': 0.033, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 800.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '62'},
        {'fromNode': '20.0', 'toNode': '27.0', 'Resistance': 0.069, 'Reactance': 0.186, 'Conductance': 0, 'Susceptance': 0.098, 'MVA_rating': 148.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '63'},
        {'fromNode': '21.0', 'toNode': '24.0', 'Resistance': 0.004, 'Reactance': 0.034, 'Conductance': 0, 'Susceptance': 0.28, 'MVA_rating': 857.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '64'},
        {'fromNode': '22.0', 'toNode': '23.0', 'Resistance': 0.052, 'Reactance': 0.111, 'Conductance': 0, 'Susceptance': 0.05, 'MVA_rating': 240.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '65'},
        {'fromNode': '23.0', 'toNode': '25.0', 'Resistance': 0.019, 'Reactance': 0.039, 'Conductance': 0, 'Susceptance': 0.018, 'MVA_rating': 677.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '66'},
        {'fromNode': '24.0', 'toNode': '319.0', 'Resistance': 0.007, 'Reactance': 0.068, 'Conductance': 0, 'Susceptance': 0.134, 'MVA_rating': 430.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '67'},
        {'fromNode': '25.0', 'toNode': '26.0', 'Resistance': 0.036, 'Reactance': 0.071, 'Conductance': 0, 'Susceptance': 0.034, 'MVA_rating': 369.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '68'},
        {'fromNode': '26.0', 'toNode': '27.0', 'Resistance': 0.045, 'Reactance': 0.12, 'Conductance': 0, 'Susceptance': 0.065, 'MVA_rating': 229.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '69'},
        {'fromNode': '26.0', 'toNode': '320.0', 'Resistance': 0.043, 'Reactance': 0.13, 'Conductance': 0, 'Susceptance': 0.014, 'MVA_rating': 215.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '70'},
        {'fromNode': '33.0', 'toNode': '34.0', 'Resistance': 0.0, 'Reactance': 0.063, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 466.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '71'},
        {'fromNode': '33.0', 'toNode': '38.0', 'Resistance': 0.0025, 'Reactance': 0.012, 'Conductance': 0, 'Susceptance': 0.013, 'MVA_rating': 2393.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '72'},
        {'fromNode': '33.0', 'toNode': '40.0', 'Resistance': 0.006, 'Reactance': 0.029, 'Conductance': 0, 'Susceptance': 0.02, 'MVA_rating': 991.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '73'},
        {'fromNode': '33.0', 'toNode': '41.0', 'Resistance': 0.007, 'Reactance': 0.043, 'Conductance': 0, 'Susceptance': 0.026, 'MVA_rating': 674.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '74'},
        {'fromNode': '34.0', 'toNode': '42.0', 'Resistance': 0.001, 'Reactance': 0.008, 'Conductance': 0, 'Susceptance': 0.042, 'MVA_rating': 3639.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '75'},
        {'fromNode': '35.0', 'toNode': '72.0', 'Resistance': 0.012, 'Reactance': 0.06, 'Conductance': 0, 'Susceptance': 0.008, 'MVA_rating': 480.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '76'},
        {'fromNode': '35.0', 'toNode': '76.0', 'Resistance': 0.006, 'Reactance': 0.014, 'Conductance': 0, 'Susceptance': 0.002, 'MVA_rating': 1926.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '77'},
        {'fromNode': '35.0', 'toNode': '77.0', 'Resistance': 0.01, 'Reactance': 0.029, 'Conductance': 0, 'Susceptance': 0.003, 'MVA_rating': 957.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '78'},
        {'fromNode': '36.0', 'toNode': '88.0', 'Resistance': 0.004, 'Reactance': 0.027, 'Conductance': 0, 'Susceptance': 0.043, 'MVA_rating': 1075.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '79'},
        {'fromNode': '37.0', 'toNode': '38.0', 'Resistance': 0.008, 'Reactance': 0.047, 'Conductance': 0, 'Susceptance': 0.008, 'MVA_rating': 616.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '80'},
        {'fromNode': '37.0', 'toNode': '40.0', 'Resistance': 0.022, 'Reactance': 0.064, 'Conductance': 0, 'Susceptance': 0.007, 'MVA_rating': 434.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '81'},
        {'fromNode': '37.0', 'toNode': '41.0', 'Resistance': 0.01, 'Reactance': 0.036, 'Conductance': 0, 'Susceptance': 0.02, 'MVA_rating': 786.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '82'},
        {'fromNode': '37.0', 'toNode': '49.0', 'Resistance': 0.017, 'Reactance': 0.081, 'Conductance': 0, 'Susceptance': 0.048, 'MVA_rating': 355.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '83'},
        {'fromNode': '37.0', 'toNode': '89.0', 'Resistance': 0.102, 'Reactance': 0.254, 'Conductance': 0, 'Susceptance': 0.033, 'MVA_rating': 108.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '84'},
        {'fromNode': '37.0', 'toNode': '90.0', 'Resistance': 0.047, 'Reactance': 0.127, 'Conductance': 0, 'Susceptance': 0.016, 'MVA_rating': 217.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '85'},
        {'fromNode': '38.0', 'toNode': '41.0', 'Resistance': 0.008, 'Reactance': 0.037, 'Conductance': 0, 'Susceptance': 0.02, 'MVA_rating': 775.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '86'},
        {'fromNode': '38.0', 'toNode': '43.0', 'Resistance': 0.032, 'Reactance': 0.087, 'Conductance': 0, 'Susceptance': 0.04, 'MVA_rating': 317.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '87'},
        {'fromNode': '39.0', 'toNode': '42.0', 'Resistance': 0.0006, 'Reactance': 0.0064, 'Conductance': 0, 'Susceptance': 0.404, 'MVA_rating': 4564.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '88'},
        {'fromNode': '40.0', 'toNode': '48.0', 'Resistance': 0.026, 'Reactance': 0.154, 'Conductance': 0, 'Susceptance': 0.022, 'MVA_rating': 188.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '89'},
        {'fromNode': '41.0', 'toNode': '42.0', 'Resistance': 0.0, 'Reactance': 0.029, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1012.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '90'},
        {'fromNode': '41.0', 'toNode': '49.0', 'Resistance': 0.065, 'Reactance': 0.191, 'Conductance': 0, 'Susceptance': 0.02, 'MVA_rating': 146.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '91'},
        {'fromNode': '41.0', 'toNode': '51.0', 'Resistance': 0.031, 'Reactance': 0.089, 'Conductance': 0, 'Susceptance': 0.036, 'MVA_rating': 312.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '92'},
        {'fromNode': '42.0', 'toNode': '46.0', 'Resistance': 0.002, 'Reactance': 0.014, 'Conductance': 0, 'Susceptance': 0.806, 'MVA_rating': 2075.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '93'},
        {'fromNode': '43.0', 'toNode': '44.0', 'Resistance': 0.026, 'Reactance': 0.072, 'Conductance': 0, 'Susceptance': 0.035, 'MVA_rating': 384.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '94'},
        {'fromNode': '43.0', 'toNode': '48.0', 'Resistance': 0.095, 'Reactance': 0.262, 'Conductance': 0, 'Susceptance': 0.032, 'MVA_rating': 106.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '95'},
        {'fromNode': '43.0', 'toNode': '53.0', 'Resistance': 0.013, 'Reactance': 0.039, 'Conductance': 0, 'Susceptance': 0.016, 'MVA_rating': 714.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '96'},
        {'fromNode': '44.0', 'toNode': '47.0', 'Resistance': 0.027, 'Reactance': 0.084, 'Conductance': 0, 'Susceptance': 0.039, 'MVA_rating': 333.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '97'},
        {'fromNode': '44.0', 'toNode': '54.0', 'Resistance': 0.028, 'Reactance': 0.084, 'Conductance': 0, 'Susceptance': 0.037, 'MVA_rating': 332.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '98'},
        {'fromNode': '45.0', 'toNode': '60.0', 'Resistance': 0.007, 'Reactance': 0.041, 'Conductance': 0, 'Susceptance': 0.312, 'MVA_rating': 706.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '99'},
        {'fromNode': '45.0', 'toNode': '74.0', 'Resistance': 0.009, 'Reactance': 0.054, 'Conductance': 0, 'Susceptance': 0.411, 'MVA_rating': 536.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '100'},
        {'fromNode': '46.0', 'toNode': '81.0', 'Resistance': 0.005, 'Reactance': 0.042, 'Conductance': 0, 'Susceptance': 0.69, 'MVA_rating': 694.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '101'},
        {'fromNode': '47.0', 'toNode': '73.0', 'Resistance': 0.052, 'Reactance': 0.145, 'Conductance': 0, 'Susceptance': 0.073, 'MVA_rating': 191.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '102'},
        {'fromNode': '47.0', 'toNode': '113.0', 'Resistance': 0.043, 'Reactance': 0.118, 'Conductance': 0, 'Susceptance': 0.013, 'MVA_rating': 234.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '103'},
        {'fromNode': '48.0', 'toNode': '107.0', 'Resistance': 0.025, 'Reactance': 0.062, 'Conductance': 0, 'Susceptance': 0.007, 'MVA_rating': 439.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '104'},
        {'fromNode': '49.0', 'toNode': '51.0', 'Resistance': 0.031, 'Reactance': 0.094, 'Conductance': 0, 'Susceptance': 0.043, 'MVA_rating': 297.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '105'},
        {'fromNode': '51.0', 'toNode': '52.0', 'Resistance': 0.037, 'Reactance': 0.109, 'Conductance': 0, 'Susceptance': 0.049, 'MVA_rating': 255.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '106'},
        {'fromNode': '52.0', 'toNode': '55.0', 'Resistance': 0.027, 'Reactance': 0.08, 'Conductance': 0, 'Susceptance': 0.036, 'MVA_rating': 348.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '107'},
        {'fromNode': '53.0', 'toNode': '54.0', 'Resistance': 0.025, 'Reactance': 0.073, 'Conductance': 0, 'Susceptance': 0.035, 'MVA_rating': 381.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '108'},
        {'fromNode': '54.0', 'toNode': '55.0', 'Resistance': 0.035, 'Reactance': 0.103, 'Conductance': 0, 'Susceptance': 0.047, 'MVA_rating': 270.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '109'},
        {'fromNode': '55.0', 'toNode': '57.0', 'Resistance': 0.065, 'Reactance': 0.169, 'Conductance': 0, 'Susceptance': 0.082, 'MVA_rating': 162.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '110'},
        {'fromNode': '57.0', 'toNode': '58.0', 'Resistance': 0.046, 'Reactance': 0.08, 'Conductance': 0, 'Susceptance': 0.036, 'MVA_rating': 318.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '111'},
        {'fromNode': '57.0', 'toNode': '63.0', 'Resistance': 0.159, 'Reactance': 0.537, 'Conductance': 0, 'Susceptance': 0.071, 'MVA_rating': 53.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '112'},
        {'fromNode': '58.0', 'toNode': '59.0', 'Resistance': 0.009, 'Reactance': 0.026, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 1067.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '113'},
        {'fromNode': '59.0', 'toNode': '61.0', 'Resistance': 0.002, 'Reactance': 0.013, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 2231.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '114'},
        {'fromNode': '60.0', 'toNode': '62.0', 'Resistance': 0.009, 'Reactance': 0.065, 'Conductance': 0, 'Susceptance': 0.485, 'MVA_rating': 447.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '115'},
        {'fromNode': '62.0', 'toNode': '64.0', 'Resistance': 0.016, 'Reactance': 0.105, 'Conductance': 0, 'Susceptance': 0.203, 'MVA_rating': 277.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '116'},
        {'fromNode': '62.0', 'toNode': '144.0', 'Resistance': 0.001, 'Reactance': 0.007, 'Conductance': 0, 'Susceptance': 0.013, 'MVA_rating': 4149.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '117'},
        {'fromNode': '63.0', 'toNode': '526.0', 'Resistance': 0.0265, 'Reactance': 0.172, 'Conductance': 0, 'Susceptance': 0.026, 'MVA_rating': 169.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '118'},
        {'fromNode': '69.0', 'toNode': '211.0', 'Resistance': 0.051, 'Reactance': 0.232, 'Conductance': 0, 'Susceptance': 0.028, 'MVA_rating': 124.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '119'},
        {'fromNode': '69.0', 'toNode': '79.0', 'Resistance': 0.051, 'Reactance': 0.157, 'Conductance': 0, 'Susceptance': 0.023, 'MVA_rating': 178.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '120'},
        {'fromNode': '70.0', 'toNode': '71.0', 'Resistance': 0.032, 'Reactance': 0.1, 'Conductance': 0, 'Susceptance': 0.062, 'MVA_rating': 280.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '121'},
        {'fromNode': '70.0', 'toNode': '528.0', 'Resistance': 0.02, 'Reactance': 0.1234, 'Conductance': 0, 'Susceptance': 0.028, 'MVA_rating': 235.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '122'},
        {'fromNode': '71.0', 'toNode': '72.0', 'Resistance': 0.036, 'Reactance': 0.131, 'Conductance': 0, 'Susceptance': 0.068, 'MVA_rating': 216.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '123'},
        {'fromNode': '71.0', 'toNode': '73.0', 'Resistance': 0.034, 'Reactance': 0.099, 'Conductance': 0, 'Susceptance': 0.047, 'MVA_rating': 281.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '124'},
        {'fromNode': '72.0', 'toNode': '77.0', 'Resistance': 0.018, 'Reactance': 0.087, 'Conductance': 0, 'Susceptance': 0.011, 'MVA_rating': 331.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '125'},
        {'fromNode': '72.0', 'toNode': '531.0', 'Resistance': 0.0256, 'Reactance': 0.193, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 151.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '126'},
        {'fromNode': '73.0', 'toNode': '76.0', 'Resistance': 0.021, 'Reactance': 0.057, 'Conductance': 0, 'Susceptance': 0.03, 'MVA_rating': 483.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '127'},
        {'fromNode': '73.0', 'toNode': '79.0', 'Resistance': 0.018, 'Reactance': 0.052, 'Conductance': 0, 'Susceptance': 0.018, 'MVA_rating': 534.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '128'},
        {'fromNode': '74.0', 'toNode': '88.0', 'Resistance': 0.004, 'Reactance': 0.027, 'Conductance': 0, 'Susceptance': 0.05, 'MVA_rating': 1075.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '129'},
        {'fromNode': '74.0', 'toNode': '562.0', 'Resistance': 0.0286, 'Reactance': 0.2013, 'Conductance': 0, 'Susceptance': 0.379, 'MVA_rating': 145.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '130'},
        {'fromNode': '76.0', 'toNode': '77.0', 'Resistance': 0.016, 'Reactance': 0.043, 'Conductance': 0, 'Susceptance': 0.004, 'MVA_rating': 640.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '131'},
        {'fromNode': '77.0', 'toNode': '78.0', 'Resistance': 0.001, 'Reactance': 0.006, 'Conductance': 0, 'Susceptance': 0.007, 'MVA_rating': 4823.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '132'},
        {'fromNode': '77.0', 'toNode': '80.0', 'Resistance': 0.014, 'Reactance': 0.07, 'Conductance': 0, 'Susceptance': 0.038, 'MVA_rating': 411.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '133'},
        {'fromNode': '77.0', 'toNode': '552.0', 'Resistance': 0.0891, 'Reactance': 0.2676, 'Conductance': 0, 'Susceptance': 0.029, 'MVA_rating': 104.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '134'},
        {'fromNode': '77.0', 'toNode': '609.0', 'Resistance': 0.0782, 'Reactance': 0.2127, 'Conductance': 0, 'Susceptance': 0.022, 'MVA_rating': 130.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '135'},
        {'fromNode': '78.0', 'toNode': '79.0', 'Resistance': 0.006, 'Reactance': 0.022, 'Conductance': 0, 'Susceptance': 0.011, 'MVA_rating': 1287.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '136'},
        {'fromNode': '78.0', 'toNode': '84.0', 'Resistance': 0.0, 'Reactance': 0.036, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 815.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '137'},
        {'fromNode': '79.0', 'toNode': '211.0', 'Resistance': 0.099, 'Reactance': 0.375, 'Conductance': 0, 'Susceptance': 0.051, 'MVA_rating': 76.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '138'},
        {'fromNode': '80.0', 'toNode': '211.0', 'Resistance': 0.022, 'Reactance': 0.107, 'Conductance': 0, 'Susceptance': 0.058, 'MVA_rating': 269.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '139'},
        {'fromNode': '81.0', 'toNode': '194.0', 'Resistance': 0.0035, 'Reactance': 0.033, 'Conductance': 0, 'Susceptance': 0.53, 'MVA_rating': 884.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '140'},
        {'fromNode': '81.0', 'toNode': '195.0', 'Resistance': 0.0035, 'Reactance': 0.033, 'Conductance': 0, 'Susceptance': 0.53, 'MVA_rating': 884.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '141'},
        {'fromNode': '85.0', 'toNode': '86.0', 'Resistance': 0.008, 'Reactance': 0.064, 'Conductance': 0, 'Susceptance': 0.128, 'MVA_rating': 455.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '142'},
        {'fromNode': '86.0', 'toNode': '87.0', 'Resistance': 0.012, 'Reactance': 0.093, 'Conductance': 0, 'Susceptance': 0.183, 'MVA_rating': 313.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '143'},
        {'fromNode': '86.0', 'toNode': '323.0', 'Resistance': 0.006, 'Reactance': 0.048, 'Conductance': 0, 'Susceptance': 0.092, 'MVA_rating': 607.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '144'},
        {'fromNode': '89.0', 'toNode': '91.0', 'Resistance': 0.047, 'Reactance': 0.119, 'Conductance': 0, 'Susceptance': 0.014, 'MVA_rating': 230.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '145'},
        {'fromNode': '90.0', 'toNode': '92.0', 'Resistance': 0.032, 'Reactance': 0.174, 'Conductance': 0, 'Susceptance': 0.024, 'MVA_rating': 166.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '146'},
        {'fromNode': '91.0', 'toNode': '94.0', 'Resistance': 0.1, 'Reactance': 0.253, 'Conductance': 0, 'Susceptance': 0.031, 'MVA_rating': 108.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '147'},
        {'fromNode': '91.0', 'toNode': '97.0', 'Resistance': 0.022, 'Reactance': 0.077, 'Conductance': 0, 'Susceptance': 0.039, 'MVA_rating': 367.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '148'},
        {'fromNode': '92.0', 'toNode': '103.0', 'Resistance': 0.019, 'Reactance': 0.144, 'Conductance': 0, 'Susceptance': 0.017, 'MVA_rating': 202.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '149'},
        {'fromNode': '92.0', 'toNode': '105.0', 'Resistance': 0.017, 'Reactance': 0.092, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 314.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '150'},
        {'fromNode': '94.0', 'toNode': '97.0', 'Resistance': 0.278, 'Reactance': 0.427, 'Conductance': 0, 'Susceptance': 0.043, 'MVA_rating': 58.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '151'},
        {'fromNode': '97.0', 'toNode': '100.0', 'Resistance': 0.022, 'Reactance': 0.053, 'Conductance': 0, 'Susceptance': 0.007, 'MVA_rating': 512.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '152'},
        {'fromNode': '97.0', 'toNode': '102.0', 'Resistance': 0.038, 'Reactance': 0.092, 'Conductance': 0, 'Susceptance': 0.012, 'MVA_rating': 295.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '153'},
        {'fromNode': '97.0', 'toNode': '103.0', 'Resistance': 0.048, 'Reactance': 0.122, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 224.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '154'},
        {'fromNode': '98.0', 'toNode': '100.0', 'Resistance': 0.024, 'Reactance': 0.064, 'Conductance': 0, 'Susceptance': 0.007, 'MVA_rating': 430.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '155'},
        {'fromNode': '98.0', 'toNode': '102.0', 'Resistance': 0.034, 'Reactance': 0.121, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 234.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '156'},
        {'fromNode': '99.0', 'toNode': '107.0', 'Resistance': 0.053, 'Reactance': 0.135, 'Conductance': 0, 'Susceptance': 0.017, 'MVA_rating': 203.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '157'},
        {'fromNode': '99.0', 'toNode': '108.0', 'Resistance': 0.002, 'Reactance': 0.004, 'Conductance': 0, 'Susceptance': 0.002, 'MVA_rating': 6559.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '158'},
        {'fromNode': '99.0', 'toNode': '109.0', 'Resistance': 0.045, 'Reactance': 0.354, 'Conductance': 0, 'Susceptance': 0.044, 'MVA_rating': 83.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '159'},
        {'fromNode': '99.0', 'toNode': '110.0', 'Resistance': 0.05, 'Reactance': 0.174, 'Conductance': 0, 'Susceptance': 0.022, 'MVA_rating': 163.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '160'},
        {'fromNode': '100.0', 'toNode': '102.0', 'Resistance': 0.016, 'Reactance': 0.038, 'Conductance': 0, 'Susceptance': 0.004, 'MVA_rating': 712.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '161'},
        {'fromNode': '102.0', 'toNode': '104.0', 'Resistance': 0.043, 'Reactance': 0.064, 'Conductance': 0, 'Susceptance': 0.027, 'MVA_rating': 381.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '162'},
        {'fromNode': '103.0', 'toNode': '105.0', 'Resistance': 0.019, 'Reactance': 0.062, 'Conductance': 0, 'Susceptance': 0.008, 'MVA_rating': 453.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '163'},
        {'fromNode': '104.0', 'toNode': '108.0', 'Resistance': 0.076, 'Reactance': 0.13, 'Conductance': 0, 'Susceptance': 0.044, 'MVA_rating': 195.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '164'},
        {'fromNode': '104.0', 'toNode': '322.0', 'Resistance': 0.044, 'Reactance': 0.124, 'Conductance': 0, 'Susceptance': 0.015, 'MVA_rating': 223.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '165'},
        {'fromNode': '105.0', 'toNode': '107.0', 'Resistance': 0.012, 'Reactance': 0.088, 'Conductance': 0, 'Susceptance': 0.011, 'MVA_rating': 331.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '166'},
        {'fromNode': '105.0', 'toNode': '110.0', 'Resistance': 0.157, 'Reactance': 0.4, 'Conductance': 0, 'Susceptance': 0.047, 'MVA_rating': 69.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '167'},
        {'fromNode': '108.0', 'toNode': '324.0', 'Resistance': 0.074, 'Reactance': 0.208, 'Conductance': 0, 'Susceptance': 0.026, 'MVA_rating': 133.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '168'},
        {'fromNode': '109.0', 'toNode': '110.0', 'Resistance': 0.07, 'Reactance': 0.184, 'Conductance': 0, 'Susceptance': 0.021, 'MVA_rating': 149.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '169'},
        {'fromNode': '109.0', 'toNode': '113.0', 'Resistance': 0.1, 'Reactance': 0.274, 'Conductance': 0, 'Susceptance': 0.031, 'MVA_rating': 101.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '170'},
        {'fromNode': '109.0', 'toNode': '114.0', 'Resistance': 0.109, 'Reactance': 0.393, 'Conductance': 0, 'Susceptance': 0.036, 'MVA_rating': 72.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '171'},
        {'fromNode': '110.0', 'toNode': '112.0', 'Resistance': 0.142, 'Reactance': 0.404, 'Conductance': 0, 'Susceptance': 0.05, 'MVA_rating': 69.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '172'},
        {'fromNode': '112.0', 'toNode': '114.0', 'Resistance': 0.017, 'Reactance': 0.042, 'Conductance': 0, 'Susceptance': 0.006, 'MVA_rating': 648.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '173'},
        {'fromNode': '115.0', 'toNode': '122.0', 'Resistance': 0.0036, 'Reactance': 0.0199, 'Conductance': 0, 'Susceptance': 0.004, 'MVA_rating': 1451.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '174'},
        {'fromNode': '116.0', 'toNode': '120.0', 'Resistance': 0.002, 'Reactance': 0.1049, 'Conductance': 0, 'Susceptance': 0.001, 'MVA_rating': 280.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '175'},
        {'fromNode': '117.0', 'toNode': '118.0', 'Resistance': 0.0001, 'Reactance': 0.0018, 'Conductance': 0, 'Susceptance': 0.017, 'MVA_rating': 9900.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '176'},
        {'fromNode': '118.0', 'toNode': '119.0', 'Resistance': 0.0, 'Reactance': 0.0271, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1083.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '177'},
        {'fromNode': '118.0', 'toNode': '1201.0', 'Resistance': 0.0, 'Reactance': 0.6163, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 48.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '178'},
        {'fromNode': '1201.0', 'toNode': '120.0', 'Resistance': 0.0, 'Reactance': -0.3697, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 80.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '179'},
        {'fromNode': '118.0', 'toNode': '121.0', 'Resistance': 0.0022, 'Reactance': 0.2915, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 101.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '180'},
        {'fromNode': '119.0', 'toNode': '120.0', 'Resistance': 0.0, 'Reactance': 0.0339, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 866.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '181'},
        {'fromNode': '119.0', 'toNode': '121.0', 'Resistance': 0.0, 'Reactance': 0.0582, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 504.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '182'},
        {'fromNode': '122.0', 'toNode': '123.0', 'Resistance': 0.0808, 'Reactance': 0.2344, 'Conductance': 0, 'Susceptance': 0.029, 'MVA_rating': 119.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '183'},
        {'fromNode': '122.0', 'toNode': '125.0', 'Resistance': 0.0965, 'Reactance': 0.3669, 'Conductance': 0, 'Susceptance': 0.054, 'MVA_rating': 78.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '184'},
        {'fromNode': '123.0', 'toNode': '124.0', 'Resistance': 0.036, 'Reactance': 0.1076, 'Conductance': 0, 'Susceptance': 0.117, 'MVA_rating': 259.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '185'},
        {'fromNode': '123.0', 'toNode': '125.0', 'Resistance': 0.0476, 'Reactance': 0.1414, 'Conductance': 0, 'Susceptance': 0.149, 'MVA_rating': 197.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '186'},
        {'fromNode': '125.0', 'toNode': '126.0', 'Resistance': 0.0006, 'Reactance': 0.0197, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1489.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '187'},
        {'fromNode': '126.0', 'toNode': '127.0', 'Resistance': 0.0059, 'Reactance': 0.0405, 'Conductance': 0, 'Susceptance': 0.25, 'MVA_rating': 717.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '188'},
        {'fromNode': '126.0', 'toNode': '129.0', 'Resistance': 0.0115, 'Reactance': 0.1106, 'Conductance': 0, 'Susceptance': 0.185, 'MVA_rating': 264.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '189'},
        {'fromNode': '126.0', 'toNode': '132.0', 'Resistance': 0.0198, 'Reactance': 0.1688, 'Conductance': 0, 'Susceptance': 0.321, 'MVA_rating': 173.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '190'},
        {'fromNode': '126.0', 'toNode': '157.0', 'Resistance': 0.005, 'Reactance': 0.05, 'Conductance': 0, 'Susceptance': 0.33, 'MVA_rating': 584.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '191'},
        {'fromNode': '126.0', 'toNode': '158.0', 'Resistance': 0.0077, 'Reactance': 0.0538, 'Conductance': 0, 'Susceptance': 0.335, 'MVA_rating': 540.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '192'},
        {'fromNode': '126.0', 'toNode': '169.0', 'Resistance': 0.0165, 'Reactance': 0.1157, 'Conductance': 0, 'Susceptance': 0.171, 'MVA_rating': 251.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '193'},
        {'fromNode': '127.0', 'toNode': '128.0', 'Resistance': 0.0059, 'Reactance': 0.0577, 'Conductance': 0, 'Susceptance': 0.095, 'MVA_rating': 506.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '194'},
        {'fromNode': '127.0', 'toNode': '134.0', 'Resistance': 0.0049, 'Reactance': 0.0336, 'Conductance': 0, 'Susceptance': 0.208, 'MVA_rating': 864.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '195'},
        {'fromNode': '127.0', 'toNode': '168.0', 'Resistance': 0.0059, 'Reactance': 0.0577, 'Conductance': 0, 'Susceptance': 0.095, 'MVA_rating': 506.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '196'},
        {'fromNode': '128.0', 'toNode': '130.0', 'Resistance': 0.0078, 'Reactance': 0.0773, 'Conductance': 0, 'Susceptance': 0.126, 'MVA_rating': 378.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '197'},
        {'fromNode': '128.0', 'toNode': '133.0', 'Resistance': 0.0026, 'Reactance': 0.0193, 'Conductance': 0, 'Susceptance': 0.03, 'MVA_rating': 1507.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '198'},
        {'fromNode': '129.0', 'toNode': '130.0', 'Resistance': 0.0076, 'Reactance': 0.0752, 'Conductance': 0, 'Susceptance': 0.122, 'MVA_rating': 389.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '199'},
        {'fromNode': '129.0', 'toNode': '133.0', 'Resistance': 0.0021, 'Reactance': 0.0186, 'Conductance': 0, 'Susceptance': 0.03, 'MVA_rating': 1568.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '200'},
        {'fromNode': '130.0', 'toNode': '132.0', 'Resistance': 0.0016, 'Reactance': 0.0164, 'Conductance': 0, 'Susceptance': 0.026, 'MVA_rating': 1781.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '201'},
        {'fromNode': '130.0', 'toNode': '151.0', 'Resistance': 0.0017, 'Reactance': 0.0165, 'Conductance': 0, 'Susceptance': 0.026, 'MVA_rating': 1769.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '202'},
        {'fromNode': '130.0', 'toNode': '167.0', 'Resistance': 0.0079, 'Reactance': 0.0793, 'Conductance': 0, 'Susceptance': 0.127, 'MVA_rating': 369.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '203'},
        {'fromNode': '130.0', 'toNode': '168.0', 'Resistance': 0.0078, 'Reactance': 0.0784, 'Conductance': 0, 'Susceptance': 0.125, 'MVA_rating': 373.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '204'},
        {'fromNode': '133.0', 'toNode': '137.0', 'Resistance': 0.0017, 'Reactance': 0.0117, 'Conductance': 0, 'Susceptance': 0.289, 'MVA_rating': 2481.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '205'},
        {'fromNode': '133.0', 'toNode': '168.0', 'Resistance': 0.0026, 'Reactance': 0.0193, 'Conductance': 0, 'Susceptance': 0.03, 'MVA_rating': 1507.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '206'},
        {'fromNode': '133.0', 'toNode': '169.0', 'Resistance': 0.0021, 'Reactance': 0.0186, 'Conductance': 0, 'Susceptance': 0.03, 'MVA_rating': 1568.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '207'},
        {'fromNode': '133.0', 'toNode': '171.0', 'Resistance': 0.0002, 'Reactance': 0.0101, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2904.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '208'},
        {'fromNode': '134.0', 'toNode': '135.0', 'Resistance': 0.0043, 'Reactance': 0.0293, 'Conductance': 0, 'Susceptance': 0.18, 'MVA_rating': 991.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '209'},
        {'fromNode': '134.0', 'toNode': '184.0', 'Resistance': 0.0039, 'Reactance': 0.0381, 'Conductance': 0, 'Susceptance': 0.258, 'MVA_rating': 766.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '210'},
        {'fromNode': '135.0', 'toNode': '136.0', 'Resistance': 0.0091, 'Reactance': 0.0623, 'Conductance': 0, 'Susceptance': 0.385, 'MVA_rating': 466.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '211'},
        {'fromNode': '136.0', 'toNode': '137.0', 'Resistance': 0.0125, 'Reactance': 0.089, 'Conductance': 0, 'Susceptance': 0.54, 'MVA_rating': 327.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '212'},
        {'fromNode': '136.0', 'toNode': '152.0', 'Resistance': 0.0056, 'Reactance': 0.039, 'Conductance': 0, 'Susceptance': 0.953, 'MVA_rating': 745.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '213'},
        {'fromNode': '137.0', 'toNode': '140.0', 'Resistance': 0.0015, 'Reactance': 0.0114, 'Conductance': 0, 'Susceptance': 0.284, 'MVA_rating': 2551.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '214'},
        {'fromNode': '137.0', 'toNode': '181.0', 'Resistance': 0.0005, 'Reactance': 0.0034, 'Conductance': 0, 'Susceptance': 0.021, 'MVA_rating': 8536.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '215'},
        {'fromNode': '137.0', 'toNode': '186.0', 'Resistance': 0.0007, 'Reactance': 0.0151, 'Conductance': 0, 'Susceptance': 0.126, 'MVA_rating': 1941.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '216'},
        {'fromNode': '137.0', 'toNode': '188.0', 'Resistance': 0.0005, 'Reactance': 0.0034, 'Conductance': 0, 'Susceptance': 0.021, 'MVA_rating': 8536.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '217'},
        {'fromNode': '139.0', 'toNode': '172.0', 'Resistance': 0.0562, 'Reactance': 0.2248, 'Conductance': 0, 'Susceptance': 0.081, 'MVA_rating': 127.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '218'},
        {'fromNode': '140.0', 'toNode': '141.0', 'Resistance': 0.012, 'Reactance': 0.0836, 'Conductance': 0, 'Susceptance': 0.123, 'MVA_rating': 348.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '219'},
        {'fromNode': '140.0', 'toNode': '142.0', 'Resistance': 0.0152, 'Reactance': 0.1132, 'Conductance': 0, 'Susceptance': 0.684, 'MVA_rating': 257.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '220'},
        {'fromNode': '140.0', 'toNode': '145.0', 'Resistance': 0.0468, 'Reactance': 0.3369, 'Conductance': 0, 'Susceptance': 0.519, 'MVA_rating': 87.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '221'},
        {'fromNode': '140.0', 'toNode': '146.0', 'Resistance': 0.043, 'Reactance': 0.3031, 'Conductance': 0, 'Susceptance': 0.463, 'MVA_rating': 96.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '222'},
        {'fromNode': '140.0', 'toNode': '147.0', 'Resistance': 0.0489, 'Reactance': 0.3492, 'Conductance': 0, 'Susceptance': 0.538, 'MVA_rating': 84.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '223'},
        {'fromNode': '140.0', 'toNode': '182.0', 'Resistance': 0.0013, 'Reactance': 0.0089, 'Conductance': 0, 'Susceptance': 0.119, 'MVA_rating': 3262.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '224'},
        {'fromNode': '141.0', 'toNode': '146.0', 'Resistance': 0.0291, 'Reactance': 0.2267, 'Conductance': 0, 'Susceptance': 0.342, 'MVA_rating': 129.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '225'},
        {'fromNode': '142.0', 'toNode': '143.0', 'Resistance': 0.006, 'Reactance': 0.057, 'Conductance': 0, 'Susceptance': 0.767, 'MVA_rating': 512.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '226'},
        {'fromNode': '143.0', 'toNode': '145.0', 'Resistance': 0.0075, 'Reactance': 0.0773, 'Conductance': 0, 'Susceptance': 0.119, 'MVA_rating': 378.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '227'},
        {'fromNode': '143.0', 'toNode': '149.0', 'Resistance': 0.0127, 'Reactance': 0.0909, 'Conductance': 0, 'Susceptance': 0.135, 'MVA_rating': 320.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '228'},
        {'fromNode': '145.0', 'toNode': '146.0', 'Resistance': 0.0085, 'Reactance': 0.0588, 'Conductance': 0, 'Susceptance': 0.087, 'MVA_rating': 494.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '229'},
        {'fromNode': '145.0', 'toNode': '149.0', 'Resistance': 0.0218, 'Reactance': 0.1511, 'Conductance': 0, 'Susceptance': 0.223, 'MVA_rating': 193.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '230'},
        {'fromNode': '146.0', 'toNode': '147.0', 'Resistance': 0.0073, 'Reactance': 0.0504, 'Conductance': 0, 'Susceptance': 0.074, 'MVA_rating': 576.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '231'},
        {'fromNode': '148.0', 'toNode': '178.0', 'Resistance': 0.0523, 'Reactance': 0.1526, 'Conductance': 0, 'Susceptance': 0.074, 'MVA_rating': 182.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '232'},
        {'fromNode': '148.0', 'toNode': '179.0', 'Resistance': 0.1371, 'Reactance': 0.3919, 'Conductance': 0, 'Susceptance': 0.076, 'MVA_rating': 71.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '233'},
        {'fromNode': '152.0', 'toNode': '153.0', 'Resistance': 0.0137, 'Reactance': 0.0957, 'Conductance': 0, 'Susceptance': 0.141, 'MVA_rating': 304.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '234'},
        {'fromNode': '153.0', 'toNode': '161.0', 'Resistance': 0.0055, 'Reactance': 0.0288, 'Conductance': 0, 'Susceptance': 0.19, 'MVA_rating': 1001.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '235'},
        {'fromNode': '154.0', 'toNode': '156.0', 'Resistance': 0.1746, 'Reactance': 0.3161, 'Conductance': 0, 'Susceptance': 0.04, 'MVA_rating': 82.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '236'},
        {'fromNode': '154.0', 'toNode': '183.0', 'Resistance': 0.0804, 'Reactance': 0.3054, 'Conductance': 0, 'Susceptance': 0.045, 'MVA_rating': 93.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '237'},
        {'fromNode': '155.0', 'toNode': '161.0', 'Resistance': 0.011, 'Reactance': 0.0568, 'Conductance': 0, 'Susceptance': 0.388, 'MVA_rating': 507.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '238'},
        {'fromNode': '157.0', 'toNode': '159.0', 'Resistance': 0.0008, 'Reactance': 0.0098, 'Conductance': 0, 'Susceptance': 0.069, 'MVA_rating': 2984.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '239'},
        {'fromNode': '158.0', 'toNode': '159.0', 'Resistance': 0.0029, 'Reactance': 0.0285, 'Conductance': 0, 'Susceptance': 0.19, 'MVA_rating': 1024.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '240'},
        {'fromNode': '158.0', 'toNode': '160.0', 'Resistance': 0.0066, 'Reactance': 0.0448, 'Conductance': 0, 'Susceptance': 0.277, 'MVA_rating': 648.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '241'},
        {'fromNode': '162.0', 'toNode': '164.0', 'Resistance': 0.0024, 'Reactance': 0.0326, 'Conductance': 0, 'Susceptance': 0.236, 'MVA_rating': 898.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '242'},
        {'fromNode': '162.0', 'toNode': '165.0', 'Resistance': 0.0018, 'Reactance': 0.0245, 'Conductance': 0, 'Susceptance': 1.662, 'MVA_rating': 1195.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '243'},
        {'fromNode': '163.0', 'toNode': '164.0', 'Resistance': 0.0044, 'Reactance': 0.0514, 'Conductance': 0, 'Susceptance': 3.597, 'MVA_rating': 569.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '244'},
        {'fromNode': '165.0', 'toNode': '166.0', 'Resistance': 0.0002, 'Reactance': 0.0123, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2385.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '245'},
        {'fromNode': '167.0', 'toNode': '169.0', 'Resistance': 0.0018, 'Reactance': 0.0178, 'Conductance': 0, 'Susceptance': 0.029, 'MVA_rating': 1640.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '246'},
        {'fromNode': '172.0', 'toNode': '173.0', 'Resistance': 0.0669, 'Reactance': 0.4843, 'Conductance': 0, 'Susceptance': 0.063, 'MVA_rating': 60.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '247'},
        {'fromNode': '172.0', 'toNode': '174.0', 'Resistance': 0.0558, 'Reactance': 0.221, 'Conductance': 0, 'Susceptance': 0.031, 'MVA_rating': 129.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '248'},
        {'fromNode': '173.0', 'toNode': '174.0', 'Resistance': 0.0807, 'Reactance': 0.3331, 'Conductance': 0, 'Susceptance': 0.049, 'MVA_rating': 86.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '249'},
        {'fromNode': '173.0', 'toNode': '175.0', 'Resistance': 0.0739, 'Reactance': 0.3071, 'Conductance': 0, 'Susceptance': 0.043, 'MVA_rating': 93.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '250'},
        {'fromNode': '173.0', 'toNode': '176.0', 'Resistance': 0.1799, 'Reactance': 0.5017, 'Conductance': 0, 'Susceptance': 0.069, 'MVA_rating': 56.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '251'},
        {'fromNode': '175.0', 'toNode': '176.0', 'Resistance': 0.0904, 'Reactance': 0.3626, 'Conductance': 0, 'Susceptance': 0.048, 'MVA_rating': 79.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '252'},
        {'fromNode': '175.0', 'toNode': '179.0', 'Resistance': 0.077, 'Reactance': 0.3092, 'Conductance': 0, 'Susceptance': 0.054, 'MVA_rating': 93.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '253'},
        {'fromNode': '176.0', 'toNode': '177.0', 'Resistance': 0.0251, 'Reactance': 0.0829, 'Conductance': 0, 'Susceptance': 0.047, 'MVA_rating': 339.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '254'},
        {'fromNode': '177.0', 'toNode': '178.0', 'Resistance': 0.0222, 'Reactance': 0.0847, 'Conductance': 0, 'Susceptance': 0.05, 'MVA_rating': 335.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '255'},
        {'fromNode': '178.0', 'toNode': '179.0', 'Resistance': 0.0498, 'Reactance': 0.1855, 'Conductance': 0, 'Susceptance': 0.029, 'MVA_rating': 153.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '256'},
        {'fromNode': '178.0', 'toNode': '180.0', 'Resistance': 0.0061, 'Reactance': 0.029, 'Conductance': 0, 'Susceptance': 0.084, 'MVA_rating': 990.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '257'},
        {'fromNode': '181.0', 'toNode': '138.0', 'Resistance': 0.0004, 'Reactance': 0.0202, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1452.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '258'},
        {'fromNode': '181.0', 'toNode': '187.0', 'Resistance': 0.0004, 'Reactance': 0.0083, 'Conductance': 0, 'Susceptance': 0.115, 'MVA_rating': 3530.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '259'},
        {'fromNode': '184.0', 'toNode': '185.0', 'Resistance': 0.0025, 'Reactance': 0.0245, 'Conductance': 0, 'Susceptance': 0.164, 'MVA_rating': 1192.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '260'},
        {'fromNode': '186.0', 'toNode': '188.0', 'Resistance': 0.0007, 'Reactance': 0.0086, 'Conductance': 0, 'Susceptance': 0.115, 'MVA_rating': 3400.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '261'},
        {'fromNode': '187.0', 'toNode': '188.0', 'Resistance': 0.0007, 'Reactance': 0.0086, 'Conductance': 0, 'Susceptance': 0.115, 'MVA_rating': 3400.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '262'},
        {'fromNode': '188.0', 'toNode': '138.0', 'Resistance': 0.0004, 'Reactance': 0.0202, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1452.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '263'},
        {'fromNode': '189.0', 'toNode': '208.0', 'Resistance': 0.033, 'Reactance': 0.095, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 292.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '264'},
        {'fromNode': '189.0', 'toNode': '209.0', 'Resistance': 0.046, 'Reactance': 0.069, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 354.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '265'},
        {'fromNode': '190.0', 'toNode': '231.0', 'Resistance': 0.0004, 'Reactance': 0.0022, 'Conductance': 0, 'Susceptance': 6.2, 'MVA_rating': 9900.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '266'},
        {'fromNode': '190.0', 'toNode': '240.0', 'Resistance': 0.0, 'Reactance': 0.0275, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1067.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '267'},
        {'fromNode': '191.0', 'toNode': '192.0', 'Resistance': 0.003, 'Reactance': 0.048, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 610.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '268'},
        {'fromNode': '192.0', 'toNode': '225.0', 'Resistance': 0.002, 'Reactance': 0.009, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 3182.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '269'},
        {'fromNode': '193.0', 'toNode': '205.0', 'Resistance': 0.045, 'Reactance': 0.063, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 379.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '270'},
        {'fromNode': '193.0', 'toNode': '208.0', 'Resistance': 0.048, 'Reactance': 0.127, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 217.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '271'},
        {'fromNode': '194.0', 'toNode': '219.0', 'Resistance': 0.0031, 'Reactance': 0.0286, 'Conductance': 0, 'Susceptance': 0.5, 'MVA_rating': 1020.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '272'},
        {'fromNode': '194.0', 'toNode': '664.0', 'Resistance': 0.0024, 'Reactance': 0.0355, 'Conductance': 0, 'Susceptance': 0.36, 'MVA_rating': 825.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '273'},
        {'fromNode': '195.0', 'toNode': '219.0', 'Resistance': 0.0031, 'Reactance': 0.0286, 'Conductance': 0, 'Susceptance': 0.5, 'MVA_rating': 1020.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '274'},
        {'fromNode': '196.0', 'toNode': '197.0', 'Resistance': 0.014, 'Reactance': 0.04, 'Conductance': 0, 'Susceptance': 0.004, 'MVA_rating': 693.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '275'},
        {'fromNode': '196.0', 'toNode': '210.0', 'Resistance': 0.03, 'Reactance': 0.081, 'Conductance': 0, 'Susceptance': 0.01, 'MVA_rating': 340.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '276'},
        {'fromNode': '197.0', 'toNode': '198.0', 'Resistance': 0.01, 'Reactance': 0.06, 'Conductance': 0, 'Susceptance': 0.009, 'MVA_rating': 483.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '277'},
        {'fromNode': '197.0', 'toNode': '211.0', 'Resistance': 0.015, 'Reactance': 0.04, 'Conductance': 0, 'Susceptance': 0.006, 'MVA_rating': 687.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '278'},
        {'fromNode': '198.0', 'toNode': '202.0', 'Resistance': 0.332, 'Reactance': 0.688, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 39.0, 'kV_base': 66.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '279'},
        {'fromNode': '198.0', 'toNode': '203.0', 'Resistance': 0.009, 'Reactance': 0.046, 'Conductance': 0, 'Susceptance': 0.025, 'MVA_rating': 626.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '280'},
        {'fromNode': '198.0', 'toNode': '210.0', 'Resistance': 0.02, 'Reactance': 0.073, 'Conductance': 0, 'Susceptance': 0.008, 'MVA_rating': 388.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '281'},
        {'fromNode': '198.0', 'toNode': '211.0', 'Resistance': 0.034, 'Reactance': 0.109, 'Conductance': 0, 'Susceptance': 0.032, 'MVA_rating': 257.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '282'},
        {'fromNode': '199.0', 'toNode': '200.0', 'Resistance': 0.076, 'Reactance': 0.135, 'Conductance': 0, 'Susceptance': 0.009, 'MVA_rating': 190.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '283'},
        {'fromNode': '199.0', 'toNode': '210.0', 'Resistance': 0.04, 'Reactance': 0.102, 'Conductance': 0, 'Susceptance': 0.005, 'MVA_rating': 268.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '284'},
        {'fromNode': '200.0', 'toNode': '210.0', 'Resistance': 0.081, 'Reactance': 0.128, 'Conductance': 0, 'Susceptance': 0.014, 'MVA_rating': 194.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '285'},
        {'fromNode': '201.0', 'toNode': '204.0', 'Resistance': 0.124, 'Reactance': 0.183, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 133.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '286'},
        {'fromNode': '203.0', 'toNode': '211.0', 'Resistance': 0.01, 'Reactance': 0.059, 'Conductance': 0, 'Susceptance': 0.008, 'MVA_rating': 491.0, 'kV_base': 115.0, 'm': 1, 'shift': 0, 'Line_id': '287'},
        {'fromNode': '204.0', 'toNode': '205.0', 'Resistance': 0.046, 'Reactance': 0.068, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 358.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '288'},
        {'fromNode': '205.0', 'toNode': '206.0', 'Resistance': 0.302, 'Reactance': 0.446, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 55.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '289'},
        {'fromNode': '206.0', 'toNode': '207.0', 'Resistance': 0.073, 'Reactance': 0.093, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 249.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '290'},
        {'fromNode': '206.0', 'toNode': '208.0', 'Resistance': 0.24, 'Reactance': 0.421, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 61.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '291'},
        {'fromNode': '212.0', 'toNode': '215.0', 'Resistance': 0.0139, 'Reactance': 0.0778, 'Conductance': 0, 'Susceptance': 0.086, 'MVA_rating': 372.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '292'},
        {'fromNode': '213.0', 'toNode': '214.0', 'Resistance': 0.0025, 'Reactance': 0.038, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 771.0, 'kV_base': 138.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '293'},
        {'fromNode': '214.0', 'toNode': '215.0', 'Resistance': 0.0017, 'Reactance': 0.0185, 'Conductance': 0, 'Susceptance': 0.02, 'MVA_rating': 1579.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '294'},
        {'fromNode': '214.0', 'toNode': '242.0', 'Resistance': 0.0015, 'Reactance': 0.0108, 'Conductance': 0, 'Susceptance': 0.002, 'MVA_rating': 2691.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '295'},
        {'fromNode': '215.0', 'toNode': '216.0', 'Resistance': 0.0045, 'Reactance': 0.0249, 'Conductance': 0, 'Susceptance': 0.026, 'MVA_rating': 1160.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '296'},
        {'fromNode': '216.0', 'toNode': '217.0', 'Resistance': 0.004, 'Reactance': 0.0497, 'Conductance': 0, 'Susceptance': 0.018, 'MVA_rating': 589.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '297'},
        {'fromNode': '217.0', 'toNode': '218.0', 'Resistance': 0.0, 'Reactance': 0.0456, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 644.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '298'},
        {'fromNode': '217.0', 'toNode': '219.0', 'Resistance': 0.0005, 'Reactance': 0.0177, 'Conductance': 0, 'Susceptance': 0.02, 'MVA_rating': 1657.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '299'},
        {'fromNode': '217.0', 'toNode': '220.0', 'Resistance': 0.0027, 'Reactance': 0.0395, 'Conductance': 0, 'Susceptance': 0.832, 'MVA_rating': 741.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '300'},
        {'fromNode': '219.0', 'toNode': '237.0', 'Resistance': 0.0003, 'Reactance': 0.0018, 'Conductance': 0, 'Susceptance': 5.2, 'MVA_rating': 9900.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '301'},
        {'fromNode': '220.0', 'toNode': '218.0', 'Resistance': 0.0037, 'Reactance': 0.0484, 'Conductance': 0, 'Susceptance': 0.43, 'MVA_rating': 605.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '302'},
        {'fromNode': '220.0', 'toNode': '221.0', 'Resistance': 0.001, 'Reactance': 0.0295, 'Conductance': 0, 'Susceptance': 0.503, 'MVA_rating': 994.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '303'},
        {'fromNode': '220.0', 'toNode': '238.0', 'Resistance': 0.0016, 'Reactance': 0.0046, 'Conductance': 0, 'Susceptance': 0.402, 'MVA_rating': 6023.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '304'},
        {'fromNode': '221.0', 'toNode': '223.0', 'Resistance': 0.0003, 'Reactance': 0.0013, 'Conductance': 0, 'Susceptance': 1.0, 'MVA_rating': 9900.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '305'},
        {'fromNode': '222.0', 'toNode': '237.0', 'Resistance': 0.0014, 'Reactance': 0.0514, 'Conductance': 0, 'Susceptance': 0.33, 'MVA_rating': 571.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '306'},
        {'fromNode': '224.0', 'toNode': '225.0', 'Resistance': 0.01, 'Reactance': 0.064, 'Conductance': 0, 'Susceptance': 0.48, 'MVA_rating': 453.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '307'},
        {'fromNode': '224.0', 'toNode': '226.0', 'Resistance': 0.0019, 'Reactance': 0.0081, 'Conductance': 0, 'Susceptance': 0.86, 'MVA_rating': 3526.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '308'},
        {'fromNode': '225.0', 'toNode': '191.0', 'Resistance': 0.001, 'Reactance': 0.061, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 481.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '309'},
        {'fromNode': '226.0', 'toNode': '231.0', 'Resistance': 0.0005, 'Reactance': 0.0212, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1384.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '310'},
        {'fromNode': '227.0', 'toNode': '231.0', 'Resistance': 0.0009, 'Reactance': 0.0472, 'Conductance': 0, 'Susceptance': 0.186, 'MVA_rating': 622.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '311'},
        {'fromNode': '228.0', 'toNode': '229.0', 'Resistance': 0.0019, 'Reactance': 0.0087, 'Conductance': 0, 'Susceptance': 1.28, 'MVA_rating': 3294.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '312'},
        {'fromNode': '228.0', 'toNode': '231.0', 'Resistance': 0.0026, 'Reactance': 0.0917, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 320.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '313'},
        {'fromNode': '228.0', 'toNode': '234.0', 'Resistance': 0.0013, 'Reactance': 0.0288, 'Conductance': 0, 'Susceptance': 0.81, 'MVA_rating': 1018.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '314'},
        {'fromNode': '229.0', 'toNode': '190.0', 'Resistance': 0.0, 'Reactance': 0.0626, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 469.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '315'},
        {'fromNode': '231.0', 'toNode': '232.0', 'Resistance': 0.0002, 'Reactance': 0.0069, 'Conductance': 0, 'Susceptance': 1.364, 'MVA_rating': 4250.0, 'kV_base': 138.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '316'},
        {'fromNode': '231.0', 'toNode': '237.0', 'Resistance': 0.0001, 'Reactance': 0.0006, 'Conductance': 0, 'Susceptance': 3.57, 'MVA_rating': 9900.0, 'kV_base': 345.0, 'm': 1, 'shift': 0, 'Line_id': '317'},
        {'fromNode': '232.0', 'toNode': '233.0', 'Resistance': 0.0017, 'Reactance': 0.0485, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 605.0, 'kV_base': 66.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '318'},
        {'fromNode': '234.0', 'toNode': '235.0', 'Resistance': 0.0002, 'Reactance': 0.0259, 'Conductance': 0, 'Susceptance': 0.144, 'MVA_rating': 1133.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '319'},
        {'fromNode': '234.0', 'toNode': '237.0', 'Resistance': 0.0006, 'Reactance': 0.0272, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1079.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '320'},
        {'fromNode': '235.0', 'toNode': '238.0', 'Resistance': 0.0002, 'Reactance': 0.0006, 'Conductance': 0, 'Susceptance': 0.8, 'MVA_rating': 9900.0, 'kV_base': 138.0, 'm': 1, 'shift': 0, 'Line_id': '321'},
        {'fromNode': '241.0', 'toNode': '237.0', 'Resistance': 0.0005, 'Reactance': 0.0154, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1904.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '322'},
        {'fromNode': '240.0', 'toNode': '281.0', 'Resistance': 0.0003, 'Reactance': 0.0043, 'Conductance': 0, 'Susceptance': 0.009, 'MVA_rating': 6805.0, 'kV_base': 230.0, 'm': 1, 'shift': 0, 'Line_id': '323'},
        {'fromNode': '242.0', 'toNode': '245.0', 'Resistance': 0.0082, 'Reactance': 0.0851, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 344.0, 'kV_base': 66.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '324'},
        {'fromNode': '242.0', 'toNode': '247.0', 'Resistance': 0.0112, 'Reactance': 0.0723, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 401.0, 'kV_base': 66.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '325'},
        {'fromNode': '243.0', 'toNode': '244.0', 'Resistance': 0.0127, 'Reactance': 0.0355, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 778.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '326'},
        {'fromNode': '243.0', 'toNode': '245.0', 'Resistance': 0.0326, 'Reactance': 0.1804, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 161.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '327'},
        {'fromNode': '244.0', 'toNode': '246.0', 'Resistance': 0.0195, 'Reactance': 0.0551, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 502.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '328'},
        {'fromNode': '245.0', 'toNode': '246.0', 'Resistance': 0.0157, 'Reactance': 0.0732, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 392.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '329'},
        {'fromNode': '245.0', 'toNode': '247.0', 'Resistance': 0.036, 'Reactance': 0.2119, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 137.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '330'},
        {'fromNode': '246.0', 'toNode': '247.0', 'Resistance': 0.0268, 'Reactance': 0.1285, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 224.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '331'},
        {'fromNode': '247.0', 'toNode': '248.0', 'Resistance': 0.0428, 'Reactance': 0.1215, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 228.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '332'},
        {'fromNode': '248.0', 'toNode': '249.0', 'Resistance': 0.0351, 'Reactance': 0.1004, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 276.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '333'},
        {'fromNode': '249.0', 'toNode': '250.0', 'Resistance': 0.0616, 'Reactance': 0.1857, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 150.0, 'kV_base': 66.0, 'm': 1, 'shift': 0, 'Line_id': '334'},
        {'fromNode': '3.0', 'toNode': '1.0', 'Resistance': 0.0, 'Reactance': 0.052, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 565.0, 'kV_base': 115.0, 'm': 0.947, 'shift': 0.0, 'Line_id': '335'},
        {'fromNode': '3.0', 'toNode': '2.0', 'Resistance': 0.0, 'Reactance': 0.052, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 565.0, 'kV_base': 115.0, 'm': 0.956, 'shift': 0.0, 'Line_id': '336'},
        {'fromNode': '3.0', 'toNode': '4.0', 'Resistance': 0.0, 'Reactance': 0.005, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 5867.0, 'kV_base': 345.0, 'm': 0.971, 'shift': 0.0, 'Line_id': '337'},
        {'fromNode': '7.0', 'toNode': '5.0', 'Resistance': 0.0, 'Reactance': 0.039, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 753.0, 'kV_base': 115.0, 'm': 0.948, 'shift': 0.0, 'Line_id': '338'},
        {'fromNode': '7.0', 'toNode': '6.0', 'Resistance': 0.0, 'Reactance': 0.039, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 753.0, 'kV_base': 115.0, 'm': 0.959, 'shift': 0.0, 'Line_id': '339'},
        {'fromNode': '10.0', 'toNode': '11.0', 'Resistance': 0.0, 'Reactance': 0.089, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 330.0, 'kV_base': 115.0, 'm': 1.046, 'shift': 0.0, 'Line_id': '340'},
        {'fromNode': '12.0', 'toNode': '10.0', 'Resistance': 0.0, 'Reactance': 0.053, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 554.0, 'kV_base': 230.0, 'm': 0.985, 'shift': 0.0, 'Line_id': '341'},
        {'fromNode': '15.0', 'toNode': '17.0', 'Resistance': 0.0194, 'Reactance': 0.0311, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 801.0, 'kV_base': 115.0, 'm': 0.9561, 'shift': 0.0, 'Line_id': '342'},
        {'fromNode': '16.0', 'toNode': '15.0', 'Resistance': 0.001, 'Reactance': 0.038, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 772.0, 'kV_base': 115.0, 'm': 0.971, 'shift': 0.0, 'Line_id': '343'},
        {'fromNode': '21.0', 'toNode': '20.0', 'Resistance': 0.0, 'Reactance': 0.014, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2096.0, 'kV_base': 115.0, 'm': 0.952, 'shift': 0.0, 'Line_id': '344'},
        {'fromNode': '24.0', 'toNode': '23.0', 'Resistance': 0.0, 'Reactance': 0.064, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 459.0, 'kV_base': 115.0, 'm': 0.943, 'shift': 0.0, 'Line_id': '345'},
        {'fromNode': '36.0', 'toNode': '35.0', 'Resistance': 0.0, 'Reactance': 0.047, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 625.0, 'kV_base': 115.0, 'm': 1.01, 'shift': 0.0, 'Line_id': '346'},
        {'fromNode': '45.0', 'toNode': '44.0', 'Resistance': 0.0, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1467.0, 'kV_base': 115.0, 'm': 1.008, 'shift': 0.0, 'Line_id': '347'},
        {'fromNode': '45.0', 'toNode': '46.0', 'Resistance': 0.0, 'Reactance': 0.021, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1397.0, 'kV_base': 345.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '348'},
        {'fromNode': '62.0', 'toNode': '61.0', 'Resistance': 0.0, 'Reactance': 0.059, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 498.0, 'kV_base': 115.0, 'm': 0.975, 'shift': 0.0, 'Line_id': '349'},
        {'fromNode': '63.0', 'toNode': '64.0', 'Resistance': 0.0, 'Reactance': 0.038, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 772.0, 'kV_base': 230.0, 'm': 1.017, 'shift': 0.0, 'Line_id': '350'},
        {'fromNode': '73.0', 'toNode': '74.0', 'Resistance': 0.0, 'Reactance': 0.0244, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1203.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '351'},
        {'fromNode': '81.0', 'toNode': '88.0', 'Resistance': 0.0, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1467.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '352'},
        {'fromNode': '85.0', 'toNode': '99.0', 'Resistance': 0.0, 'Reactance': 0.048, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 612.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '353'},
        {'fromNode': '86.0', 'toNode': '102.0', 'Resistance': 0.0, 'Reactance': 0.048, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 612.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '354'},
        {'fromNode': '87.0', 'toNode': '94.0', 'Resistance': 0.0, 'Reactance': 0.046, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 638.0, 'kV_base': 115.0, 'm': 1.015, 'shift': 0.0, 'Line_id': '355'},
        {'fromNode': '114.0', 'toNode': '207.0', 'Resistance': 0.0, 'Reactance': 0.149, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 197.0, 'kV_base': 66.0, 'm': 0.967, 'shift': 0.0, 'Line_id': '356'},
        {'fromNode': '116.0', 'toNode': '124.0', 'Resistance': 0.0052, 'Reactance': 0.0174, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1616.0, 'kV_base': 115.0, 'm': 1.01, 'shift': 0.0, 'Line_id': '357'},
        {'fromNode': '121.0', 'toNode': '115.0', 'Resistance': 0.0, 'Reactance': 0.028, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1048.0, 'kV_base': 115.0, 'm': 1.05, 'shift': 0.0, 'Line_id': '358'},
        {'fromNode': '122.0', 'toNode': '157.0', 'Resistance': 0.0005, 'Reactance': 0.0195, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1504.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '359'},
        {'fromNode': '130.0', 'toNode': '131.0', 'Resistance': 0.0, 'Reactance': 0.018, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1630.0, 'kV_base': 230.0, 'm': 1.0522, 'shift': 0.0, 'Line_id': '360'},
        {'fromNode': '130.0', 'toNode': '150.0', 'Resistance': 0.0, 'Reactance': 0.014, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2096.0, 'kV_base': 230.0, 'm': 1.0522, 'shift': 0.0, 'Line_id': '361'},
        {'fromNode': '132.0', 'toNode': '170.0', 'Resistance': 0.001, 'Reactance': 0.0402, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 730.0, 'kV_base': 115.0, 'm': 1.05, 'shift': 0.0, 'Line_id': '362'},
        {'fromNode': '141.0', 'toNode': '174.0', 'Resistance': 0.0024, 'Reactance': 0.0603, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 487.0, 'kV_base': 115.0, 'm': 0.975, 'shift': 0.0, 'Line_id': '363'},
        {'fromNode': '142.0', 'toNode': '175.0', 'Resistance': 0.0024, 'Reactance': 0.0498, 'Conductance': 0, 'Susceptance': -0.087, 'MVA_rating': 589.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '364'},
        {'fromNode': '143.0', 'toNode': '144.0', 'Resistance': 0.0, 'Reactance': 0.0833, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 353.0, 'kV_base': 230.0, 'm': 1.035, 'shift': 0.0, 'Line_id': '365'},
        {'fromNode': '143.0', 'toNode': '148.0', 'Resistance': 0.0013, 'Reactance': 0.0371, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 791.0, 'kV_base': 230.0, 'm': 0.9565, 'shift': 0.0, 'Line_id': '366'},
        {'fromNode': '145.0', 'toNode': '180.0', 'Resistance': 0.0005, 'Reactance': 0.0182, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1612.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '367'},
        {'fromNode': '151.0', 'toNode': '170.0', 'Resistance': 0.001, 'Reactance': 0.0392, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 749.0, 'kV_base': 115.0, 'm': 1.05, 'shift': 0.0, 'Line_id': '368'},
        {'fromNode': '153.0', 'toNode': '183.0', 'Resistance': 0.0027, 'Reactance': 0.0639, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 459.0, 'kV_base': 115.0, 'm': 1.073, 'shift': 0.0, 'Line_id': '369'},
        {'fromNode': '155.0', 'toNode': '156.0', 'Resistance': 0.0008, 'Reactance': 0.0256, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1146.0, 'kV_base': 115.0, 'm': 1.05, 'shift': 0.0, 'Line_id': '370'},
        {'fromNode': '159.0', 'toNode': '117.0', 'Resistance': 0.0, 'Reactance': 0.016, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1834.0, 'kV_base': 115.0, 'm': 1.0506, 'shift': 0.0, 'Line_id': '371'},
        {'fromNode': '160.0', 'toNode': '124.0', 'Resistance': 0.0012, 'Reactance': 0.0396, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 741.0, 'kV_base': 115.0, 'm': 0.975, 'shift': 0.0, 'Line_id': '372'},
        {'fromNode': '163.0', 'toNode': '137.0', 'Resistance': 0.0013, 'Reactance': 0.0384, 'Conductance': 0, 'Susceptance': -0.057, 'MVA_rating': 764.0, 'kV_base': 230.0, 'm': 0.98, 'shift': 0.0, 'Line_id': '373'},
        {'fromNode': '164.0', 'toNode': '155.0', 'Resistance': 0.0009, 'Reactance': 0.0231, 'Conductance': 0, 'Susceptance': -0.033, 'MVA_rating': 1269.0, 'kV_base': 230.0, 'm': 0.956, 'shift': 0.0, 'Line_id': '374'},
        {'fromNode': '182.0', 'toNode': '139.0', 'Resistance': 0.0003, 'Reactance': 0.0131, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2239.0, 'kV_base': 115.0, 'm': 1.05, 'shift': 0.0, 'Line_id': '375'},
        {'fromNode': '189.0', 'toNode': '210.0', 'Resistance': 0.0, 'Reactance': 0.252, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 117.0, 'kV_base': 115.0, 'm': 1.03, 'shift': 0.0, 'Line_id': '376'},
        {'fromNode': '193.0', 'toNode': '196.0', 'Resistance': 0.0, 'Reactance': 0.237, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 124.0, 'kV_base': 115.0, 'm': 1.03, 'shift': 0.0, 'Line_id': '377'},
        {'fromNode': '195.0', 'toNode': '212.0', 'Resistance': 0.0008, 'Reactance': 0.0366, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 802.0, 'kV_base': 138.0, 'm': 0.985, 'shift': 0.0, 'Line_id': '378'},
        {'fromNode': '200.0', 'toNode': '248.0', 'Resistance': 0.0, 'Reactance': 0.22, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 134.0, 'kV_base': 66.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '379'},
        {'fromNode': '201.0', 'toNode': '69.0', 'Resistance': 0.0, 'Reactance': 0.098, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 300.0, 'kV_base': 115.0, 'm': 1.03, 'shift': 0.0, 'Line_id': '380'},
        {'fromNode': '202.0', 'toNode': '211.0', 'Resistance': 0.0, 'Reactance': 0.128, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 230.0, 'kV_base': 115.0, 'm': 1.01, 'shift': 0.0, 'Line_id': '381'},
        {'fromNode': '204.0', 'toNode': '2040.0', 'Resistance': 0.02, 'Reactance': 0.204, 'Conductance': 0, 'Susceptance': -0.012, 'MVA_rating': 144.0, 'kV_base': 115.0, 'm': 1.05, 'shift': 0.0, 'Line_id': '382'},
        {'fromNode': '209.0', 'toNode': '198.0', 'Resistance': 0.026, 'Reactance': 0.211, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 138.0, 'kV_base': 115.0, 'm': 1.03, 'shift': 0.0, 'Line_id': '383'},
        {'fromNode': '211.0', 'toNode': '212.0', 'Resistance': 0.003, 'Reactance': 0.0122, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2335.0, 'kV_base': 138.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '384'},
        {'fromNode': '218.0', 'toNode': '219.0', 'Resistance': 0.001, 'Reactance': 0.0354, 'Conductance': 0, 'Susceptance': -0.01, 'MVA_rating': 829.0, 'kV_base': 345.0, 'm': 0.97, 'shift': 0.0, 'Line_id': '385'},
        {'fromNode': '223.0', 'toNode': '224.0', 'Resistance': 0.0012, 'Reactance': 0.0195, 'Conductance': 0, 'Susceptance': -0.364, 'MVA_rating': 1502.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '386'},
        {'fromNode': '229.0', 'toNode': '230.0', 'Resistance': 0.001, 'Reactance': 0.0332, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 884.0, 'kV_base': 20.0, 'm': 1.02, 'shift': 0.0, 'Line_id': '387'},
        {'fromNode': '234.0', 'toNode': '236.0', 'Resistance': 0.0005, 'Reactance': 0.016, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1833.0, 'kV_base': 20.0, 'm': 1.07, 'shift': 0.0, 'Line_id': '388'},
        {'fromNode': '238.0', 'toNode': '239.0', 'Resistance': 0.0005, 'Reactance': 0.016, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1833.0, 'kV_base': 138.0, 'm': 1.02, 'shift': 0.0, 'Line_id': '389'},
        {'fromNode': '196.0', 'toNode': '2040.0', 'Resistance': 0.0001, 'Reactance': 0.02, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1467.0, 'kV_base': 115.0, 'm': 1.0, 'shift': -0.19896753472735357, 'Line_id': '390'},
        {'fromNode': '119.0', 'toNode': '1190.0', 'Resistance': 0.001, 'Reactance': 0.023, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1275.0, 'kV_base': 86.0, 'm': 1.0223, 'shift': 0.0, 'Line_id': '391'},
        {'fromNode': '120.0', 'toNode': '1200.0', 'Resistance': 0.0, 'Reactance': 0.023, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1276.0, 'kV_base': 86.0, 'm': 0.9284, 'shift': 0.0, 'Line_id': '392'},
        {'fromNode': '7002.0', 'toNode': '2.0', 'Resistance': 0.001, 'Reactance': 0.0146, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2005.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '393'},
        {'fromNode': '7003.0', 'toNode': '3.0', 'Resistance': 0.0, 'Reactance': 0.01054, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2783.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '394'},
        {'fromNode': '7061.0', 'toNode': '61.0', 'Resistance': 0.0, 'Reactance': 0.0238, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1233.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '395'},
        {'fromNode': '7062.0', 'toNode': '62.0', 'Resistance': 0.0, 'Reactance': 0.03214, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 913.0, 'kV_base': 230.0, 'm': 0.95, 'shift': 0.0, 'Line_id': '396'},
        {'fromNode': '7166.0', 'toNode': '166.0', 'Resistance': 0.0, 'Reactance': 0.0154, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1905.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '397'},
        {'fromNode': '7024.0', 'toNode': '24.0', 'Resistance': 0.0, 'Reactance': 0.0289, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1015.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '398'},
        {'fromNode': '7001.0', 'toNode': '1.0', 'Resistance': 0.0, 'Reactance': 0.01953, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1502.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '399'},
        {'fromNode': '7130.0', 'toNode': '130.0', 'Resistance': 0.0, 'Reactance': 0.0193, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1520.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '400'},
        {'fromNode': '7011.0', 'toNode': '11.0', 'Resistance': 0.0, 'Reactance': 0.01923, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1526.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '401'},
        {'fromNode': '7023.0', 'toNode': '23.0', 'Resistance': 0.0, 'Reactance': 0.023, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1276.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '402'},
        {'fromNode': '7049.0', 'toNode': '49.0', 'Resistance': 0.0, 'Reactance': 0.0124, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 2366.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '403'},
        {'fromNode': '7139.0', 'toNode': '139.0', 'Resistance': 0.0, 'Reactance': 0.0167, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1757.0, 'kV_base': 115.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '404'},
        {'fromNode': '7012.0', 'toNode': '12.0', 'Resistance': 0.0, 'Reactance': 0.0312, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 941.0, 'kV_base': 230.0, 'm': 1.0, 'shift': 0.0, 'Line_id': '405'},
        {'fromNode': '7017.0', 'toNode': '17.0', 'Resistance': 0.0, 'Reactance': 0.01654, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 1774.0, 'kV_base': 115.0, 'm': 0.942, 'shift': 0.0, 'Line_id': '406'},
        {'fromNode': '7039.0', 'toNode': '39.0', 'Resistance': 0.0, 'Reactance': 0.03159, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 929.0, 'kV_base': 345.0, 'm': 0.965, 'shift': 0.0, 'Line_id': '407'},
        {'fromNode': '7057.0', 'toNode': '57.0', 'Resistance': 0.0, 'Reactance': 0.05347, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 549.0, 'kV_base': 115.0, 'm': 0.95, 'shift': 0.0, 'Line_id': '408'},
        {'fromNode': '7044.0', 'toNode': '44.0', 'Resistance': 0.0, 'Reactance': 0.18181, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 162.0, 'kV_base': 115.0, 'm': 0.942, 'shift': 0.0, 'Line_id': '409'},
        {'fromNode': '7055.0', 'toNode': '55.0', 'Resistance': 0.0, 'Reactance': 0.19607, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 150.0, 'kV_base': 115.0, 'm': 0.942, 'shift': 0.0, 'Line_id': '410'},
        {'fromNode': '7071.0', 'toNode': '71.0', 'Resistance': 0.0, 'Reactance': 0.06896, 'Conductance': 0, 'Susceptance': 0.0, 'MVA_rating': 426.0, 'kV_base': 115.0, 'm': 0.9565, 'shift': 0.0, 'Line_id': '411'}
    ]
    lines_AC = pd.DataFrame(lines_AC_data)

    nodes_DC = None

    lines_DC = None

    Converters_ACDC = None

    
    # Create the grid
    [grid, res] = pyf.Create_grid_from_data(S_base, nodes_AC, lines_AC, nodes_DC, lines_DC, Converters_ACDC, data_in_pu=True)
    
    # Assign Price Zones to Nodes
    for index, row in nodes_AC.iterrows():
        node_name = nodes_AC.at[index, 'Node_id']
        price_zone = nodes_AC.at[index, 'PZ']
        ACDC = 'AC'
        if price_zone is not None:
            pyf.assign_nodeToPrice_Zone(grid, node_name, ACDC, price_zone)
    
    # Add Generators
    pyf.add_gen(grid, '8.0', '1', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=10.0, MVArmin=-10.0, PsetMW=0.0, QsetMVA=0.0)
    pyf.add_gen(grid, '10.0', '2', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=20.0, MVArmin=-20.0, PsetMW=0.0, QsetMVA=0.0)
    pyf.add_gen(grid, '20.0', '3', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=20.0, MVArmin=-20.0, PsetMW=0.0, QsetMVA=0.0)
    pyf.add_gen(grid, '63.0', '4', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=25.0, MVArmin=-25.0, PsetMW=0.0, QsetMVA=0.0)
    pyf.add_gen(grid, '76.0', '5', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=35.0, MVArmin=12.0, PsetMW=0.0, QsetMVA=23.5)
    pyf.add_gen(grid, '84.0', '6', price_zone_link=False, lf=22.409835, qf=0.0, MWmax=1448.0, MWmin=0.0, MVArmax=240.0, MVArmin=-240.0, PsetMW=724.0, QsetMVA=0.0)
    pyf.add_gen(grid, '91.0', '7', price_zone_link=False, lf=46.100054, qf=0.0, MWmax=185.0, MWmin=0.0, MVArmax=93.0, MVArmin=-11.0, PsetMW=92.5, QsetMVA=41.0)
    pyf.add_gen(grid, '92.0', '8', price_zone_link=False, lf=54.164939, qf=0.0, MWmax=360.0, MWmin=0.0, MVArmax=153.0, MVArmin=-153.0, PsetMW=180.0, QsetMVA=0.0)
    pyf.add_gen(grid, '98.0', '9', price_zone_link=False, lf=27.238416, qf=0.0, MWmax=93.0, MWmin=0.0, MVArmax=47.0, MVArmin=-30.0, PsetMW=46.5, QsetMVA=8.5)
    pyf.add_gen(grid, '108.0', '10', price_zone_link=False, lf=38.430722, qf=0.0, MWmax=197.0, MWmin=0.0, MVArmax=77.0, MVArmin=-24.0, PsetMW=98.5, QsetMVA=26.5)
    pyf.add_gen(grid, '119.0', '11', price_zone_link=False, lf=7.698908, qf=0.0, MWmax=2041.0, MWmin=0.0, MVArmax=1021.0000000000001, MVArmin=-500.0, PsetMW=1020.5, QsetMVA=260.5)
    pyf.add_gen(grid, '124.0', '12', price_zone_link=False, lf=26.754128, qf=0.0, MWmax=321.0, MWmin=0.0, MVArmax=120.0, MVArmin=-60.0, PsetMW=160.5, QsetMVA=30.0)
    pyf.add_gen(grid, '125.0', '13', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=200.0, MVArmin=-25.0, PsetMW=0.0, QsetMVA=87.5)
    pyf.add_gen(grid, '138.0', '14', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=350.0, MVArmin=-125.0, PsetMW=0.0, QsetMVA=112.5)
    pyf.add_gen(grid, '141.0', '15', price_zone_link=False, lf=30.597916, qf=0.0, MWmax=297.0, MWmin=0.0, MVArmax=75.0, MVArmin=-50.0, PsetMW=148.5, QsetMVA=12.5)
    pyf.add_gen(grid, '143.0', '16', price_zone_link=False, lf=30.758666, qf=0.0, MWmax=800.0, MWmin=0.0, MVArmax=300.0, MVArmin=-100.0, PsetMW=400.0, QsetMVA=100.0)
    pyf.add_gen(grid, '146.0', '17', price_zone_link=False, lf=19.292281, qf=0.0, MWmax=91.0, MWmin=0.0, MVArmax=35.0, MVArmin=-15.0, PsetMW=45.5, QsetMVA=10.0)
    pyf.add_gen(grid, '147.0', '18', price_zone_link=False, lf=49.861855, qf=0.0, MWmax=320.0, MWmin=0.0, MVArmax=100.0, MVArmin=-50.0, PsetMW=160.0, QsetMVA=25.0)
    pyf.add_gen(grid, '149.0', '19', price_zone_link=False, lf=35.842363, qf=0.0, MWmax=277.0, MWmin=0.0, MVArmax=50.0, MVArmin=-25.0, PsetMW=138.5, QsetMVA=12.5)
    pyf.add_gen(grid, '152.0', '20', price_zone_link=False, lf=34.955965, qf=0.0, MWmax=606.0, MWmin=0.0, MVArmax=175.0, MVArmin=-50.0, PsetMW=303.0, QsetMVA=62.5)
    pyf.add_gen(grid, '153.0', '21', price_zone_link=False, lf=22.762722, qf=0.0, MWmax=242.0, MWmin=0.0, MVArmax=90.0, MVArmin=-50.0, PsetMW=121.0, QsetMVA=20.0)
    pyf.add_gen(grid, '156.0', '22', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=15.0, MVArmin=-10.0, PsetMW=0.0, QsetMVA=2.5)
    pyf.add_gen(grid, '170.0', '23', price_zone_link=False, lf=35.375974, qf=0.0, MWmax=330.0, MWmin=0.0, MVArmax=90.0, MVArmin=-40.0, PsetMW=165.0, QsetMVA=25.0)
    pyf.add_gen(grid, '171.0', '24', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=150.0, MVArmin=-50.0, PsetMW=0.0, QsetMVA=50.0)
    pyf.add_gen(grid, '176.0', '25', price_zone_link=False, lf=33.868524, qf=0.0, MWmax=278.0, MWmin=0.0, MVArmax=90.0, MVArmin=-45.0, PsetMW=139.0, QsetMVA=22.5)
    pyf.add_gen(grid, '177.0', '26', price_zone_link=False, lf=29.031265, qf=0.0, MWmax=125.0, MWmin=0.0, MVArmax=35.0, MVArmin=-15.0, PsetMW=62.5, QsetMVA=10.0)
    pyf.add_gen(grid, '185.0', '27', price_zone_link=False, lf=28.391517, qf=0.0, MWmax=250.0, MWmin=0.0, MVArmax=80.0, MVArmin=-50.0, PsetMW=125.0, QsetMVA=15.0)
    pyf.add_gen(grid, '186.0', '28', price_zone_link=False, lf=1.000203, qf=0.0, MWmax=2465.0, MWmin=0.0, MVArmax=400.0, MVArmin=-100.0, PsetMW=1232.5, QsetMVA=150.0)
    pyf.add_gen(grid, '187.0', '29', price_zone_link=False, lf=27.278538, qf=0.0, MWmax=1623.9999999999998, MWmin=0.0, MVArmax=400.0, MVArmin=-100.0, PsetMW=811.9999999999999, QsetMVA=150.0)
    pyf.add_gen(grid, '190.0', '30', price_zone_link=False, lf=49.15571, qf=0.0, MWmax=515.0, MWmin=0.0, MVArmax=258.0, MVArmin=-258.0, PsetMW=257.5, QsetMVA=0.0)
    pyf.add_gen(grid, '191.0', '31', price_zone_link=False, lf=24.062655, qf=0.0, MWmax=2140.0, MWmin=0.0, MVArmax=1000.0, MVArmin=-1000.0, PsetMW=1070.0, QsetMVA=0.0)
    pyf.add_gen(grid, '198.0', '32', price_zone_link=False, lf=31.016664, qf=0.0, MWmax=430.0, MWmin=0.0, MVArmax=215.0, MVArmin=-215.0, PsetMW=215.0, QsetMVA=0.0)
    pyf.add_gen(grid, '213.0', '33', price_zone_link=False, lf=49.721799, qf=0.0, MWmax=288.0, MWmin=0.0, MVArmax=144.0, MVArmin=-144.0, PsetMW=144.0, QsetMVA=0.0)
    pyf.add_gen(grid, '220.0', '34', price_zone_link=False, lf=21.291665, qf=0.0, MWmax=459.99999999999994, MWmin=0.0, MVArmax=60.0, MVArmin=-60.0, PsetMW=229.99999999999997, QsetMVA=0.0)
    pyf.add_gen(grid, '221.0', '35', price_zone_link=False, lf=31.437031, qf=0.0, MWmax=554.0, MWmin=0.0, MVArmax=277.0, MVArmin=-277.0, PsetMW=277.0, QsetMVA=0.0)
    pyf.add_gen(grid, '222.0', '36', price_zone_link=False, lf=50.842869, qf=0.0, MWmax=523.0, MWmin=0.0, MVArmax=262.0, MVArmin=-262.0, PsetMW=261.5, QsetMVA=0.0)
    pyf.add_gen(grid, '227.0', '37', price_zone_link=False, lf=28.01046, qf=0.0, MWmax=436.00000000000006, MWmin=0.0, MVArmax=218.00000000000003, MVArmin=-218.00000000000003, PsetMW=218.00000000000003, QsetMVA=0.0)
    pyf.add_gen(grid, '230.0', '38', price_zone_link=False, lf=39.828572, qf=0.0, MWmax=458.0, MWmin=0.0, MVArmax=229.0, MVArmin=-229.0, PsetMW=229.0, QsetMVA=0.0)
    pyf.add_gen(grid, '233.0', '39', price_zone_link=False, lf=35.650779, qf=0.0, MWmax=341.0, MWmin=0.0, MVArmax=171.0, MVArmin=-171.0, PsetMW=170.5, QsetMVA=0.0)
    pyf.add_gen(grid, '236.0', '40', price_zone_link=False, lf=32.621266, qf=0.0, MWmax=881.0, MWmin=0.0, MVArmax=300.0, MVArmin=-300.0, PsetMW=440.5, QsetMVA=0.0)
    pyf.add_gen(grid, '238.0', '41', price_zone_link=False, lf=39.657688, qf=0.0, MWmax=267.0, MWmin=0.0, MVArmax=134.0, MVArmin=-134.0, PsetMW=133.5, QsetMVA=0.0)
    pyf.add_gen(grid, '239.0', '42', price_zone_link=False, lf=41.223312, qf=0.0, MWmax=621.0, MWmin=0.0, MVArmax=311.0, MVArmin=-311.0, PsetMW=310.5, QsetMVA=0.0)
    pyf.add_gen(grid, '241.0', '43', price_zone_link=False, lf=39.064721, qf=0.0, MWmax=1127.0, MWmin=0.0, MVArmax=564.0, MVArmin=-564.0, PsetMW=563.5, QsetMVA=0.0)
    pyf.add_gen(grid, '242.0', '44', price_zone_link=False, lf=116.939409, qf=0.0, MWmax=212.0, MWmin=0.0, MVArmax=100.0, MVArmin=40.0, PsetMW=106.0, QsetMVA=70.0)
    pyf.add_gen(grid, '243.0', '45', price_zone_link=False, lf=34.336562, qf=0.0, MWmax=342.0, MWmin=0.0, MVArmax=80.0, MVArmin=40.0, PsetMW=171.0, QsetMVA=60.0)
    pyf.add_gen(grid, '7001.0', '46', price_zone_link=False, lf=38.274878, qf=0.0, MWmax=568.0, MWmin=0.0, MVArmax=210.0, MVArmin=-210.0, PsetMW=284.0, QsetMVA=0.0)
    pyf.add_gen(grid, '7002.0', '47', price_zone_link=False, lf=23.466762, qf=0.0, MWmax=670.0, MWmin=0.0, MVArmax=280.0, MVArmin=-280.0, PsetMW=335.0, QsetMVA=0.0)
    pyf.add_gen(grid, '7003.0', '48', price_zone_link=False, lf=7.509959, qf=0.0, MWmax=1401.0, MWmin=0.0, MVArmax=420.0, MVArmin=-420.0, PsetMW=700.5, QsetMVA=0.0)
    pyf.add_gen(grid, '7011.0', '49', price_zone_link=False, lf=26.682965, qf=0.0, MWmax=668.0, MWmin=0.0, MVArmax=100.0, MVArmin=-100.0, PsetMW=334.0, QsetMVA=0.0)
    pyf.add_gen(grid, '7012.0', '50', price_zone_link=False, lf=28.434868, qf=0.0, MWmax=626.0, MWmin=0.0, MVArmax=224.00000000000003, MVArmin=-224.00000000000003, PsetMW=313.0, QsetMVA=0.0)
    pyf.add_gen(grid, '7017.0', '51', price_zone_link=False, lf=35.126651, qf=0.0, MWmax=490.00000000000006, MWmin=0.0, MVArmax=245.00000000000003, MVArmin=0.0, PsetMW=245.00000000000003, QsetMVA=122.50000000000001)
    pyf.add_gen(grid, '7023.0', '52', price_zone_link=False, lf=51.062397, qf=0.0, MWmax=220.00000000000003, MWmin=0.0, MVArmax=110.00000000000001, MVArmin=0.0, PsetMW=110.00000000000001, QsetMVA=55.00000000000001)
    pyf.add_gen(grid, '7024.0', '53', price_zone_link=False, lf=29.992015, qf=0.0, MWmax=543.0, MWmin=0.0, MVArmax=224.00000000000003, MVArmin=-224.00000000000003, PsetMW=271.5, QsetMVA=0.0)
    pyf.add_gen(grid, '7039.0', '54', price_zone_link=False, lf=103.987496, qf=0.0, MWmax=580.0, MWmin=0.0, MVArmax=200.0, MVArmin=-200.0, PsetMW=290.0, QsetMVA=0.0)
    pyf.add_gen(grid, '7044.0', '55', price_zone_link=False, lf=38.825689, qf=0.0, MWmax=132.0, MWmin=0.0, MVArmax=42.0, MVArmin=0.0, PsetMW=66.0, QsetMVA=21.0)
    pyf.add_gen(grid, '7049.0', '56', price_zone_link=False, lf=6.843072, qf=0.0, MWmax=718.0, MWmin=0.0, MVArmax=10.0, MVArmin=0.0, PsetMW=359.0, QsetMVA=5.0)
    pyf.add_gen(grid, '7055.0', '57', price_zone_link=False, lf=29.772562, qf=0.0, MWmax=158.0, MWmin=0.0, MVArmax=25.0, MVArmin=0.0, PsetMW=79.0, QsetMVA=12.5)
    pyf.add_gen(grid, '7057.0', '58', price_zone_link=False, lf=62.427103, qf=0.0, MWmax=334.0, MWmin=0.0, MVArmax=90.0, MVArmin=-90.0, PsetMW=167.0, QsetMVA=0.0)
    pyf.add_gen(grid, '7061.0', '59', price_zone_link=False, lf=20.811515, qf=0.0, MWmax=1757.0, MWmin=0.0, MVArmax=150.0, MVArmin=-150.0, PsetMW=878.5, QsetMVA=0.0)
    pyf.add_gen(grid, '7062.0', '60', price_zone_link=False, lf=26.50789, qf=0.0, MWmax=568.0, MWmin=0.0, MVArmax=150.0, MVArmin=0.0, PsetMW=284.0, QsetMVA=75.0)
    pyf.add_gen(grid, '7071.0', '61', price_zone_link=False, lf=31.587517, qf=0.0, MWmax=202.99999999999997, MWmin=0.0, MVArmax=87.0, MVArmin=0.0, PsetMW=101.49999999999999, QsetMVA=43.5)
    pyf.add_gen(grid, '7130.0', '62', price_zone_link=False, lf=29.759924, qf=0.0, MWmax=1726.0000000000002, MWmin=0.0, MVArmax=600.0, MVArmin=-100.0, PsetMW=863.0000000000001, QsetMVA=250.0)
    pyf.add_gen(grid, '7139.0', '63', price_zone_link=False, lf=36.179055, qf=0.0, MWmax=2025.0, MWmin=0.0, MVArmax=325.0, MVArmin=-125.0, PsetMW=1012.5, QsetMVA=100.0)
    pyf.add_gen(grid, '7166.0', '64', price_zone_link=False, lf=36.908367, qf=0.0, MWmax=624.0, MWmin=0.0, MVArmax=300.0, MVArmin=-200.0, PsetMW=312.0, QsetMVA=50.0)
    pyf.add_gen(grid, '9002.0', '65', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=2.0, MVArmin=-2.0, PsetMW=0.0, QsetMVA=0.0)
    pyf.add_gen(grid, '9051.0', '66', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=17.35, MVArmin=-17.35, PsetMW=0.0, QsetMVA=0.0)
    pyf.add_gen(grid, '9053.0', '67', price_zone_link=False, lf=0.0, qf=0.0, MWmax=0.0, MWmin=0.0, MVArmax=12.83, MVArmin=-12.8, PsetMW=0.0, QsetMVA=0.015)
    pyf.add_gen(grid, '9054.0', '68', price_zone_link=False, lf=32.961473, qf=0.0, MWmax=84.0, MWmin=0.0, MVArmax=38.0, MVArmin=-38.0, PsetMW=42.0, QsetMVA=0.0)
    pyf.add_gen(grid, '9055.0', '69', price_zone_link=False, lf=94.758209, qf=0.0, MWmax=37.0, MWmin=0.0, MVArmax=6.0, MVArmin=-6.0, PsetMW=18.5, QsetMVA=0.0)
    
    
    # Add Renewable Source Zones

    
    # Add Renewable Sources

    
    # Return the grid
    return grid,res
